@@ -12,7 +12,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cscanner_c[] = "$Header: /cvsroot/swig/SWIG/Source/CParse/cscanner.c,v 1.8 2004/02/20 22:51:47 cheetah Exp $";
+char cvsroot_cscanner_c[] = "$Header: /cvsroot/swig/SWIG/Source/CParse/cscanner.c,v 1.11 2004/11/15 20:51:47 marcelomatus Exp $";
 
 #include "cparse.h"
 #include "parser.h"
@@ -47,7 +47,7 @@ static  int    num_brace = 0;
 static  int    last_brace = 0;
 static  int    last_id = 0;
 static  int    rename_active = 0;
-        int    cparse_cplusplus;
+        int    cparse_cplusplus = 0;
 
 /* -----------------------------------------------------------------------------
  * Swig_cparse_cplusplus()
@@ -1200,6 +1200,10 @@ int yylex(void) {
 	    if (strcmp(yytext,"class") == 0) {
 	      Swig_warning(WARN_PARSE_CLASS_KEYWORD,cparse_file,cparse_line, "class keyword used, but not in C++ mode.\n");
 	    }
+	    if (strcmp(yytext,"complex") == 0) {
+	      yylval.type = NewSwigType(T_COMPLEX);
+	      return(TYPE_COMPLEX);
+	    }
 	  }
 	  
 	  /* Objective-C keywords */
@@ -1234,6 +1238,7 @@ int yylex(void) {
 	  /* Ignored keywords */
 	  
 	  if (strcmp(yytext,"volatile") == 0) return(VOLATILE);
+	  if (strcmp(yytext,"register") == 0) return(REGISTER);
 	  
 	  /* SWIG directives */
 	} else {
