@@ -55,89 +55,91 @@
      TYPE_VOID = 281,
      TYPE_SIGNED = 282,
      TYPE_BOOL = 283,
-     TYPE_TYPEDEF = 284,
-     TYPE_RAW = 285,
-     LPAREN = 286,
-     RPAREN = 287,
-     COMMA = 288,
-     SEMI = 289,
-     EXTERN = 290,
-     INIT = 291,
-     LBRACE = 292,
-     RBRACE = 293,
-     PERIOD = 294,
-     CONST_QUAL = 295,
-     VOLATILE = 296,
-     STRUCT = 297,
-     UNION = 298,
-     EQUAL = 299,
-     SIZEOF = 300,
-     MODULE = 301,
-     LBRACKET = 302,
-     RBRACKET = 303,
-     ILLEGAL = 304,
-     CONSTANT = 305,
-     NAME = 306,
-     RENAME = 307,
-     NAMEWARN = 308,
-     EXTEND = 309,
-     PRAGMA = 310,
-     FEATURE = 311,
-     VARARGS = 312,
-     ENUM = 313,
-     CLASS = 314,
-     TYPENAME = 315,
-     PRIVATE = 316,
-     PUBLIC = 317,
-     PROTECTED = 318,
-     COLON = 319,
-     STATIC = 320,
-     VIRTUAL = 321,
-     FRIEND = 322,
-     THROW = 323,
-     CATCH = 324,
-     USING = 325,
-     NAMESPACE = 326,
-     NATIVE = 327,
-     INLINE = 328,
-     TYPEMAP = 329,
-     EXCEPT = 330,
-     ECHO = 331,
-     APPLY = 332,
-     CLEAR = 333,
-     SWIGTEMPLATE = 334,
-     FRAGMENT = 335,
-     WARN = 336,
-     LESSTHAN = 337,
-     GREATERTHAN = 338,
-     MODULO = 339,
-     DELETE_KW = 340,
-     TYPES = 341,
-     PARMS = 342,
-     NONID = 343,
-     DSTAR = 344,
-     DCNOT = 345,
-     TEMPLATE = 346,
-     OPERATOR = 347,
-     COPERATOR = 348,
-     PARSETYPE = 349,
-     PARSEPARM = 350,
-     CAST = 351,
-     LOR = 352,
-     LAND = 353,
-     OR = 354,
-     XOR = 355,
-     AND = 356,
-     RSHIFT = 357,
-     LSHIFT = 358,
-     MINUS = 359,
-     PLUS = 360,
-     SLASH = 361,
-     STAR = 362,
-     LNOT = 363,
-     NOT = 364,
-     UMINUS = 365,
-     DCOLON = 366
+     TYPE_COMPLEX = 284,
+     TYPE_TYPEDEF = 285,
+     TYPE_RAW = 286,
+     LPAREN = 287,
+     RPAREN = 288,
+     COMMA = 289,
+     SEMI = 290,
+     EXTERN = 291,
+     INIT = 292,
+     LBRACE = 293,
+     RBRACE = 294,
+     PERIOD = 295,
+     CONST_QUAL = 296,
+     VOLATILE = 297,
+     REGISTER = 298,
+     STRUCT = 299,
+     UNION = 300,
+     EQUAL = 301,
+     SIZEOF = 302,
+     MODULE = 303,
+     LBRACKET = 304,
+     RBRACKET = 305,
+     ILLEGAL = 306,
+     CONSTANT = 307,
+     NAME = 308,
+     RENAME = 309,
+     NAMEWARN = 310,
+     EXTEND = 311,
+     PRAGMA = 312,
+     FEATURE = 313,
+     VARARGS = 314,
+     ENUM = 315,
+     CLASS = 316,
+     TYPENAME = 317,
+     PRIVATE = 318,
+     PUBLIC = 319,
+     PROTECTED = 320,
+     COLON = 321,
+     STATIC = 322,
+     VIRTUAL = 323,
+     FRIEND = 324,
+     THROW = 325,
+     CATCH = 326,
+     USING = 327,
+     NAMESPACE = 328,
+     NATIVE = 329,
+     INLINE = 330,
+     TYPEMAP = 331,
+     EXCEPT = 332,
+     ECHO = 333,
+     APPLY = 334,
+     CLEAR = 335,
+     SWIGTEMPLATE = 336,
+     FRAGMENT = 337,
+     WARN = 338,
+     LESSTHAN = 339,
+     GREATERTHAN = 340,
+     MODULO = 341,
+     DELETE_KW = 342,
+     TYPES = 343,
+     PARMS = 344,
+     NONID = 345,
+     DSTAR = 346,
+     DCNOT = 347,
+     TEMPLATE = 348,
+     OPERATOR = 349,
+     COPERATOR = 350,
+     PARSETYPE = 351,
+     PARSEPARM = 352,
+     CAST = 353,
+     LOR = 354,
+     LAND = 355,
+     OR = 356,
+     XOR = 357,
+     AND = 358,
+     RSHIFT = 359,
+     LSHIFT = 360,
+     MINUS = 361,
+     PLUS = 362,
+     SLASH = 363,
+     STAR = 364,
+     LNOT = 365,
+     NOT = 366,
+     UMINUS = 367,
+     DCOLON = 368
    };
 #endif
 #define ID 258
@@ -166,95 +168,97 @@
 #define TYPE_VOID 281
 #define TYPE_SIGNED 282
 #define TYPE_BOOL 283
-#define TYPE_TYPEDEF 284
-#define TYPE_RAW 285
-#define LPAREN 286
-#define RPAREN 287
-#define COMMA 288
-#define SEMI 289
-#define EXTERN 290
-#define INIT 291
-#define LBRACE 292
-#define RBRACE 293
-#define PERIOD 294
-#define CONST_QUAL 295
-#define VOLATILE 296
-#define STRUCT 297
-#define UNION 298
-#define EQUAL 299
-#define SIZEOF 300
-#define MODULE 301
-#define LBRACKET 302
-#define RBRACKET 303
-#define ILLEGAL 304
-#define CONSTANT 305
-#define NAME 306
-#define RENAME 307
-#define NAMEWARN 308
-#define EXTEND 309
-#define PRAGMA 310
-#define FEATURE 311
-#define VARARGS 312
-#define ENUM 313
-#define CLASS 314
-#define TYPENAME 315
-#define PRIVATE 316
-#define PUBLIC 317
-#define PROTECTED 318
-#define COLON 319
-#define STATIC 320
-#define VIRTUAL 321
-#define FRIEND 322
-#define THROW 323
-#define CATCH 324
-#define USING 325
-#define NAMESPACE 326
-#define NATIVE 327
-#define INLINE 328
-#define TYPEMAP 329
-#define EXCEPT 330
-#define ECHO 331
-#define APPLY 332
-#define CLEAR 333
-#define SWIGTEMPLATE 334
-#define FRAGMENT 335
-#define WARN 336
-#define LESSTHAN 337
-#define GREATERTHAN 338
-#define MODULO 339
-#define DELETE_KW 340
-#define TYPES 341
-#define PARMS 342
-#define NONID 343
-#define DSTAR 344
-#define DCNOT 345
-#define TEMPLATE 346
-#define OPERATOR 347
-#define COPERATOR 348
-#define PARSETYPE 349
-#define PARSEPARM 350
-#define CAST 351
-#define LOR 352
-#define LAND 353
-#define OR 354
-#define XOR 355
-#define AND 356
-#define RSHIFT 357
-#define LSHIFT 358
-#define MINUS 359
-#define PLUS 360
-#define SLASH 361
-#define STAR 362
-#define LNOT 363
-#define NOT 364
-#define UMINUS 365
-#define DCOLON 366
+#define TYPE_COMPLEX 284
+#define TYPE_TYPEDEF 285
+#define TYPE_RAW 286
+#define LPAREN 287
+#define RPAREN 288
+#define COMMA 289
+#define SEMI 290
+#define EXTERN 291
+#define INIT 292
+#define LBRACE 293
+#define RBRACE 294
+#define PERIOD 295
+#define CONST_QUAL 296
+#define VOLATILE 297
+#define REGISTER 298
+#define STRUCT 299
+#define UNION 300
+#define EQUAL 301
+#define SIZEOF 302
+#define MODULE 303
+#define LBRACKET 304
+#define RBRACKET 305
+#define ILLEGAL 306
+#define CONSTANT 307
+#define NAME 308
+#define RENAME 309
+#define NAMEWARN 310
+#define EXTEND 311
+#define PRAGMA 312
+#define FEATURE 313
+#define VARARGS 314
+#define ENUM 315
+#define CLASS 316
+#define TYPENAME 317
+#define PRIVATE 318
+#define PUBLIC 319
+#define PROTECTED 320
+#define COLON 321
+#define STATIC 322
+#define VIRTUAL 323
+#define FRIEND 324
+#define THROW 325
+#define CATCH 326
+#define USING 327
+#define NAMESPACE 328
+#define NATIVE 329
+#define INLINE 330
+#define TYPEMAP 331
+#define EXCEPT 332
+#define ECHO 333
+#define APPLY 334
+#define CLEAR 335
+#define SWIGTEMPLATE 336
+#define FRAGMENT 337
+#define WARN 338
+#define LESSTHAN 339
+#define GREATERTHAN 340
+#define MODULO 341
+#define DELETE_KW 342
+#define TYPES 343
+#define PARMS 344
+#define NONID 345
+#define DSTAR 346
+#define DCNOT 347
+#define TEMPLATE 348
+#define OPERATOR 349
+#define COPERATOR 350
+#define PARSETYPE 351
+#define PARSEPARM 352
+#define CAST 353
+#define LOR 354
+#define LAND 355
+#define OR 356
+#define XOR 357
+#define AND 358
+#define RSHIFT 359
+#define LSHIFT 360
+#define MINUS 361
+#define PLUS 362
+#define SLASH 363
+#define STAR 364
+#define LNOT 365
+#define NOT 366
+#define UMINUS 367
+#define DCOLON 368
 
 
 
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 874 "parser.y"
+#line 1206 "parser.y"
 typedef union YYSTYPE {
   char  *id;
   List  *bases;
@@ -265,6 +269,7 @@ typedef union YYSTYPE {
     String *qualifier;
     String *bitfield;
     Parm   *throws;
+    String *throw;
   } dtype;
   struct {
     char *type;
@@ -278,6 +283,7 @@ typedef union YYSTYPE {
     ParmList  *parms;
     short      have_parms;
     ParmList  *throws;
+    String    *throw;
   } decl;
   Parm         *tparms;
   struct {
@@ -296,7 +302,7 @@ typedef union YYSTYPE {
   Node         *node;
 } YYSTYPE;
 /* Line 1249 of yacc.c.  */
-#line 299 "y.tab.h"
+#line 305 "y.tab.h"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
