@@ -9,7 +9,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_templ_c[] = "$Header: /cvsroot/swig/SWIG/Source/CParse/templ.c,v 1.11 2004/03/17 08:01:06 mmatus Exp $";
+char cvsroot_templ_c[] = "$Header: /cvsroot/swig/SWIG/Source/CParse/templ.c,v 1.19 2004/12/05 10:15:25 marcelomatus Exp $";
 
 #include "swig.h"
 #include "cparse.h"
@@ -21,6 +21,7 @@ static void add_parms(ParmList *p, List *patchlist, List *typelist) {
     SwigType *ty = Getattr(p,"type");
     SwigType *val = Getattr(p,"value");
     Append(typelist,ty);
+    Append(typelist,val);
     Append(patchlist,val);
     p = nextSibling(p);
   }
@@ -91,13 +92,17 @@ cparse_template_expand(Node *n, String *tname, String *rname, String *templatear
   } else if (Strcmp(nodeType(n),"class") == 0) {
     /* Patch base classes */
     {
-      List *bases = Getattr(n,"baselist");
-      if (bases) {
-	int i;
-	for (i = 0; i < Len(bases); i++) {
-	  String *name = Copy(Getitem(bases,i));
-	  Setitem(bases,i,name);
-	  Append(typelist,name);
+      char *baselists[] = {"baselist","protectedbaselist","privatebaselist"};
+      int b = 0;
+      for (b = 0; b < 3; ++b) {
+	List *bases = Getattr(n,baselists[b]);
+	if (bases) {
+	  int i;
+	  for (i = 0; i < Len(bases); i++) {
+	    String *name = Copy(Getitem(bases,i));
+	    Setitem(bases,i,name);
+	    Append(typelist,name);
+	  }
 	}
       }
     }
@@ -209,7 +214,7 @@ String *partial_arg(String *s, String *p) {
 }
 
 int
-Swig_cparse_template_expand(Node *n, String *rname, ParmList *tparms) {
+Swig_cparse_template_expand(Node *n, String *rname, ParmList *tparms, Symtab *tscope) {
   List *patchlist, *cpatchlist, *typelist;
   String *templateargs;
   String *tname;
@@ -280,7 +285,7 @@ Swig_cparse_template_expand(Node *n, String *rname, ParmList *tparms) {
   {
     Parm *tp = Getattr(n,"templateparms");
     Parm *p  = tparms;
-    /*    Printf(stdout,"%s\n", ParmList_str(tp)); */
+    /*    Printf(stdout,"%s\n", ParmList_str_defaultargs(tp)); */
 
     if (tp) {
       while (p && tp) {
@@ -292,13 +297,12 @@ Swig_cparse_template_expand(Node *n, String *rname, ParmList *tparms) {
 	tydef = Getattr(p,"typedef");
 
 	if (name) {
-	  if (!value) {
-	    value = Getattr(p,"type");
-	    valuestr = SwigType_str(value,0);
-	  } else {
-	    valuestr = SwigType_namestr(value);
+	  if (!value) value = Getattr(p,"type");
+	  if (SwigType_istemplate(value)) {
+	    value = Swig_cparse_template_deftype(value, 0);
 	  }
-	  /*	  Printf(stdout,"valuestr = '%s'\n", valuestr); */
+	  
+	  valuestr = SwigType_str(value,0);
 	  assert(value);
 	  /* Need to patch default arguments */
 	  {
@@ -364,7 +368,7 @@ Swig_cparse_template_expand(Node *n, String *rname, ParmList *tparms) {
     if (bases) {
       Iterator b;
       for (b = First(bases); b.item; b = Next(b)) {
-	String *qn = Swig_symbol_type_qualify(b.item,0);
+	String *qn = Swig_symbol_type_qualify(b.item,tscope);
 	Clear(b.item);
 	Append(b.item,qn);
       }
@@ -380,30 +384,142 @@ Swig_cparse_template_expand(Node *n, String *rname, ParmList *tparms) {
 }
 
 /* -----------------------------------------------------------------------------
- * cparse_template_locate()
+ * Swig_cparse_template_defargs()
+ *
+ * Apply default arg from generic template default args 
+ * ----------------------------------------------------------------------------- */
+
+
+void
+Swig_cparse_template_defargs(Parm *parms, Parm *targs, Symtab *tscope) {
+  if (Len(parms) < Len(targs)) {
+    Parm *lp = parms;
+    Parm *p = lp;
+    Parm *tp = targs;
+    while(p && tp) {
+      p = nextSibling(p);
+      tp = nextSibling(tp);
+      if (p) lp = p;
+    }
+    while (tp) {
+      String *value = Getattr(tp,"value");
+      if (value) {
+	Parm *cp;
+	Parm *ta = targs;
+	Parm *p = parms;
+	SwigType *nt  = Swig_symbol_typedef_reduce(value,tscope);
+	while(p && ta) {
+	  String *name = Getattr(ta,"name");
+	  String *value = Getattr(p,"value");
+	  if (!value) value = Getattr(p,"type");
+	  Replace(nt, name, value, DOH_REPLACE_ID);
+	  p = nextSibling(p);
+	  ta = nextSibling(ta);
+	}
+	cp = NewParm(Swig_symbol_type_qualify(nt,tscope),0);
+	set_nextSibling(lp,cp);
+	lp = cp;
+	tp = nextSibling(tp);
+	Delete(nt);
+      } else {
+	tp = 0;
+      }
+    }
+  }
+}
+
+/* -----------------------------------------------------------------------------
+ * Swig_cparse_template_deftype()
+ *
+ * Apply default args to generic template type
+ * ----------------------------------------------------------------------------- */
+String*
+Swig_cparse_template_deftype(SwigType *type, Symtab *tscope) {
+  String *prefix  = SwigType_prefix(type);
+  String *base    = SwigType_base(type);
+  String *tprefix = SwigType_templateprefix(base);
+  String *targs   = SwigType_templateargs(base);
+  String *tsuffix = SwigType_templatesuffix(base);
+  ParmList *tparms = SwigType_function_parms(targs);
+  Node *tempn = Swig_symbol_clookup_local(tprefix,tscope);
+  if (tempn) {
+    ParmList *tnargs = Getattr(tempn,"templateparms");
+    Parm *p;
+    /* Printf(stderr,"deftype type %s %s %s\n", tprefix, targs, tsuffix);*/
+    Append(tprefix,"<(");
+    Swig_cparse_template_defargs(tparms, tnargs,tscope);
+    p = tparms;
+    while (p) {
+      SwigType *ptype = Getattr(p,"type");
+      SwigType *ttr = Swig_symbol_typedef_reduce(ptype ? ptype : Getattr(p,"value") ,tscope);
+      SwigType *ttq = Swig_symbol_type_qualify(ttr,tscope);
+      if (SwigType_istemplate(ttq)) {
+	SwigType *t = Swig_cparse_template_deftype(ttq, tscope);
+	Delete(ttq);
+	ttq = t;
+      }	
+      Append(tprefix,ttq);
+      p = nextSibling(p);
+      if (p) Putc(',',tprefix);
+      Delete(ttr);
+      Delete(ttq);
+    }
+    Append(tprefix,")>");
+    Append(tprefix,tsuffix);
+    Append(prefix,tprefix);
+    /* Printf(stderr,"deftype default %s \n", tprefix); */
+    type = Copy(prefix);
+  } else {
+    type = Copy(type);
+  }
+  
+  Delete(prefix);
+  Delete(base);
+  Delete(tprefix);
+  Delete(tsuffix);
+  Delete(targs);
+  Delete(tparms);
+  return type;
+}
+
+/* -----------------------------------------------------------------------------
+ * template_locate()
  *
  * Search for a template that matches name with given parameters.
  * ----------------------------------------------------------------------------- */
 
-Node *
-Swig_cparse_template_locate(String *name, Parm *tparms) {
+static Node *
+template_locate(String *name, Parm *tparms, Symtab *tscope) {
   Node   *n;
   String *tname, *rname = 0;
   Node   *templ;
   List   *mpartials = 0;
   Parm   *p;
   Parm   *parms;
+  Parm   *targs;
 
   tname = NewString(name);
   parms = CopyParmList(tparms);
 
+  /* Search for generic template */
+  templ = Swig_symbol_clookup(name,0);
+
+  /* Add default values from generic template */
+  if (templ) {
+    targs = Getattr(templ,"templateparms");
+    Swig_cparse_template_defargs(parms, targs, tscope);
+  }
+  
+
+  /* reduce the typedef */
   p = parms;
   while (p) {
     SwigType *ty = Getattr(p,"type");
     if (ty) {
-      SwigType *nt = Swig_symbol_typedef_reduce(ty,0);
-      nt = Swig_symbol_type_qualify(nt,0);
+      SwigType *rt = Swig_symbol_typedef_reduce(ty,tscope);
+      SwigType *nt = Swig_symbol_type_qualify(rt,tscope);
       Setattr(p,"type",nt);
+      Delete(rt);
     }
     p = nextSibling(p);
   }
@@ -435,10 +551,7 @@ Swig_cparse_template_locate(String *name, Parm *tparms) {
 	  return 0;        /* Found a match, but it's not a template of any kind. */
       }
   }
-
-  /* Search for generic template */
-  templ = Swig_symbol_clookup_local(name,0);
-
+  
   /* Search for partial specialization. 
      Example: template<typename T> class name<T *> { ... } */
 
@@ -452,8 +565,8 @@ Swig_cparse_template_locate(String *name, Parm *tparms) {
     if (t) {
       String *tbase = SwigType_base(t);
       t = SwigType_default(t);
+      Replaceid(t,"enum SWIGTYPE",tbase);
       Replaceid(t,"SWIGTYPE",tbase);
-      Replaceid(t,"SWIGENUM",tbase);
       Printf(rname,"%s",t);
       Delete(t);
     } else {
@@ -542,7 +655,7 @@ Swig_cparse_template_locate(String *name, Parm *tparms) {
   Delete(rname);
   Delete(mpartials);
   if ((template_debug) && (n)) {
-    Printf(stdout,"Node: %x\n", n);
+    Printf(stdout,"Node: %p\n", n);
       Swig_print_node(n);
   }
   Delete(parms);
@@ -550,4 +663,58 @@ Swig_cparse_template_locate(String *name, Parm *tparms) {
 }
 
 
+/* -----------------------------------------------------------------------------
+ * Swig_cparse_template_locate()
+ *
+ * Search for a template that matches name with given parameters.
+ * For templated classes finds the specialized template should there be one.
+ * For templated functions finds the unspecialized template even if a specialized
+ * template exists.
+ * ----------------------------------------------------------------------------- */
+
+Node *
+Swig_cparse_template_locate(String *name, Parm *tparms, Symtab *tscope) {
+  Node *n = 0;
+  n = template_locate(name, tparms, tscope); /* this function does what we want for templated classes */
+
+  if (n) {
+    int isclass = 0;
+    assert(Strcmp(nodeType(n),"template") == 0);
+    isclass = (Strcmp(Getattr(n,"templatetype"),"class") == 0);
+    if (!isclass) {
+      /* If not a templated class we must have a templated function.
+         The template found is not necessarily the one we want when dealing with templated
+         functions. We don't want any specialized templated functions as they won't have
+         the default parameters. Lets look for the unspecialized template. Also make sure
+         the number of template parameters is correct as it is possible to overload a
+         templated function with different numbers of template parameters. */
+
+      if (template_debug) {
+        Printf(stdout,"    Not a templated class, seeking most appropriate templated function\n");
+      }
+
+      n = Swig_symbol_clookup_local(name,0);
+      while (n) {
+        Parm *tparmsfound = Getattr(n,"templateparms");
+        if (ParmList_len(tparms) == ParmList_len(tparmsfound)) {
+          /* successful match */
+          break;
+        }
+        /* repeat until we find a match with correct number of templated parameters */
+        n = Getattr(n,"sym:nextSibling");
+      }
+ 
+      if (!n) {
+        Swig_error(cparse_file, cparse_line, "Template '%s' undefined.\n", name);
+      }
+
+      if ((template_debug) && (n)) {
+        Printf(stdout,"Templated function found: %p\n", n);
+          Swig_print_node(n);
+      }
+    }
+  }
+
+  return n;
+}
 
