@@ -10,7 +10,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_fio_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/fio.c,v 1.4 2004/01/15 22:46:04 cheetah Exp $";
+char cvsroot_fio_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/fio.c,v 1.5 2004/10/18 02:22:15 marcelomatus Exp $";
 
 #include "dohint.h"
 
@@ -105,7 +105,7 @@ DohvPrintf(DOH *so, const char *format, va_list ap)
   int   nbytes = 0;
   char  encoder[128], *ec = 0;
 
-  memset (newformat, 0, sizeof (char *));
+  memset (newformat, 0, sizeof (newformat));
 
   while (*p) {
     switch(state) {
