@@ -10,7 +10,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_string_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/string.c,v 1.10 2004/01/22 22:42:13 cheetah Exp $";
+char cvsroot_string_c[] = "$Header: /cvsroot/swig/SWIG/Source/DOH/string.c,v 1.13 2004/09/27 20:17:36 wsfulton Exp $";
 
 #include "dohint.h"
 
@@ -111,25 +111,26 @@ String_len(DOH *so) {
 static int
 String_cmp(DOH *so1, DOH *so2)
 {
-  String *s1, *s2;
-  char *c1, *c2;
-  int  maxlen,i;
-  s1 = (String *) ObjData(so1);
-  s2 = (String *) ObjData(so2);
-  maxlen = s1->len;
-  if (s2->len < maxlen) maxlen = s2->len;
-  c1 = s1->str;
-  c2 = s2->str;
-  for (i = 0; i < maxlen; i++,c1++,c2++) {
-    if (*c1 != *c2) break;
+  String * s1 = (String *) ObjData(so1);
+  String * s2 = (String *) ObjData(so2);
+  register int len = s1->len;
+  register char *c1 = s1->str;
+  register char *c2 = s2->str;
+#if 1
+  /* this should be faster */
+  if (len != s2->len) {
+    return (len < s2->len) ? -1 : 1;
+  } else {  
+    return strncmp(c1, c2, len);
   }
-  if (i < maxlen) {
-    if (*c1 < *c2) return -1;
-    else return 1;
+#else
+  register char *ce  = c1 + len;
+  if (len != s2->len) return (len < s2->len ) ? -1 : 1;
+  for (; c1 != ce ; ++c1, ++c2) {
+    if (*c1 != *c2) return ((*c1 < *c2) ? -1 : 1);
   }
-  if (s1->len == s2->len) return 0;
-  if (s1->len > s2->len) return 1;
-  return -1;
+  return 0;
+#endif
 }
 
 /* -----------------------------------------------------------------------------
