@@ -16,7 +16,7 @@
  * See the file LICENSE for information on usage and redistribution.	
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_allocate_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/allocate.cxx,v 1.23 2004/05/29 23:17:15 marcelomatus Exp $";
+char cvsroot_allocate_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/allocate.cxx,v 1.36 2004/12/07 23:32:11 marcelomatus Exp $";
 
 #include "swigmod.h"
 
@@ -60,15 +60,13 @@ static int check_implemented(Node *n) {
 
 class Allocate : public Dispatcher {
   Node  *inclass;
-  enum AccessMode { PUBLIC, PRIVATE, PROTECTED };
-  AccessMode cplus_mode;
   int extendmode;
 
   /* Checks if a virtual function is the same as inherited from the bases */
   int function_is_defined_in_bases(Node *c, Node *bases) {
     Node *b, *temp;
-    String *name, *type, *local_decl, *base_decl;
-    SwigType *base_type, *local_type;
+    String *name, *type, *local_decl, *base_decl, *base_access;
+    SwigType *base_type, *local_type, *local_access;
     
     if (!bases)
       return 0;
@@ -76,6 +74,7 @@ class Allocate : public Dispatcher {
     name = Getattr(c, "name");
     type = Getattr(c, "type");
     local_decl = Getattr(c, "decl");
+    local_access = Getattr(c, "access");
     if (local_decl) {
       local_decl = SwigType_typedef_resolve_all(local_decl);
     } else {
@@ -90,6 +89,7 @@ class Allocate : public Dispatcher {
       while (temp) {
 	base_decl = Getattr(temp, "decl");
 	base_type = Getattr(temp, "type");
+	base_access = Getattr(temp, "access");
 	if (base_decl && base_type) {
 	  base_decl = SwigType_typedef_resolve_all(base_decl);
 	  base_type = SwigType_typedef_resolve_all(base_type);
@@ -103,18 +103,29 @@ class Allocate : public Dispatcher {
             // is, not the virtual method definition in a base class
             Setattr(c, "storage", "virtual");
             Setattr(c, "virtual:derived", "1"); 
-            if (Strcmp(local_type, base_type) == 0) {
-	      // if the types are the same, then we can attemp
+            if ((Strcmp(local_type, base_type) == 0)) {
+	      // if the types and access are the same, then we can attemp
 	      // to eliminate the derived virtual method.
-              if (virtual_elimination_mode) Setattr(c,"feature:ignore", "1");
+              if (virtual_elimination_mode) {
+		const char *la = local_access ? Char(local_access) : "";
+		const char *ba = base_access ? Char(base_access) : "";
+		if (strcmp(la, ba) == 0) Setattr(c,"feature:ignore", "1");
+	      }
             } else {
-	      // if the types are different, we record the original
-	      // virtual base type in case some language needs it.
-              Setattr(c, "virtual:type", Getattr(temp, "type"));
+	      // if the types are different, we record the base type
+	      // those languages that need to know about covariant return types
+              SwigType *ty = NewString(Getattr(temp,"type"));
+              SwigType_push(ty,Getattr(temp,"decl"));
+              if (SwigType_isqualifier(ty)) {
+                SwigType_pop(ty);
+              }
+              Delete(SwigType_pop_function(ty));
+
+              Setattr(c, "virtual:type", ty);
             }
 
-            Delete(base_decl);
-            Delete(base_type);
+	    Delete(base_decl);
+	    Delete(base_type);
             Delete(local_decl);
             Delete(local_type);
             return 1;
@@ -129,7 +140,7 @@ class Allocate : public Dispatcher {
     Delete(local_type);
     for (int j = 0; j < Len(bases); j++) {
       b = Getitem(bases,j);
-      if (function_is_defined_in_bases(c, Getattr(b, "bases")))
+      if (function_is_defined_in_bases(c, Getattr(b, "allbases")))
 	return 1;
     }
     return 0;
@@ -141,7 +152,7 @@ class Allocate : public Dispatcher {
     Node *c, *bases; /* bases is the closest ancestors of class n */
     int defined = 0;
 
-    bases = Getattr(n, "bases");
+    bases = Getattr(n, "allbases");
 
     if (!bases) return 0;
 
@@ -164,7 +175,7 @@ class Allocate : public Dispatcher {
     Node *bases; /* bases is the closest ancestors of classnode */
     int defined = 0;
 
-    bases = Getattr(classnode, "bases");
+    bases = Getattr(classnode, "allbases");
     if (!bases) return 0;
 
     //if (checkAttribute(member, "storage", "virtual"))
@@ -205,52 +216,28 @@ class Allocate : public Dispatcher {
 	String *base_decl = Getattr(nn,"decl");
 	if (base_decl) base_decl = SwigType_typedef_resolve_all(base_decl);
 	if (Strstr(name,"~")) continue;   /* Don't care about destructors */
-    /*
-	int implemented = 0;
-	Node *dn = Swig_symbol_clookup_local(name,0);
-	if (!dn) {
-	  Printf(stdout,"node: %x '%s'. base: %x '%s'. member '%s'\n", n, Getattr(n,"name"), base, Getattr(base,"name"), name);
-	}
-	assert(dn != 0);   // Assertion of doom
-	*/
+
 	if (SwigType_isfunction(base_decl)) {
 	  search_decl = SwigType_pop_function(base_decl);
 	}
 	Node *dn = Swig_symbol_clookup_local_check(name,0,check_implemented);
 	Delete(search_decl);
 	Delete(base_decl);
-	/*
-	while (dn && !implemented) {
-	  String *local_decl = Getattr(dn,"decl");
-	  if (local_decl) local_decl = SwigType_typedef_resolve_all(local_decl);
-	  if (local_decl && !Strcmp(local_decl, base_decl)) {
-	    if (Getattr(dn,"abstract")) return 1;
-	    implemented++;
-	  }
-	  Delete(local_decl);
-	  dn = Getattr(dn,"csym:nextSibling");
-	}
-	*/
-
-
-
-	/*	if (!implemented && (Getattr(nn,"abstract"))) {
-	  return 1;
-	}
-	*/
 
 	if (!dn) {
-	  Setattr(n,"abstract:firstnode",nn);
-	  return 1;
+	  List *abstract = Getattr(n,"abstract");
+	  if (!abstract) {
+	    abstract = NewList();
+	    Setattr(n,"abstract",abstract);
+	  } else {
+	    if (!Getattr(n,"abstract:firstnode"))
+	      Setattr(n,"abstract:firstnode",nn);
+	  }
+	  Append(abstract,nn);
 	}
-	/*
-	if (dn && (Getattr(dn,"abstract"))) {
-	  return 1;
-	} 
-	*/
       }
     }
-    List *bases = Getattr(base,"bases");
+    List *bases = Getattr(base,"allbases");
     if (!bases) return 0;
     for (int i = 0; i < Len(bases); i++) {
       if (is_abstract_inherit(n,Getitem(bases,i))) {
@@ -263,26 +250,28 @@ class Allocate : public Dispatcher {
 
   /* Grab methods used by smart pointers */
 
-  List *smart_pointer_methods(Node *cls, List *methods, int isconst) {
+  List *smart_pointer_methods(Node *cls, List *methods, int isconst,
+			      String *classname=0) {
     if (!methods) {
       methods = NewList();
     }
     
     Node *c = firstChild(cls);
     String *kind = Getattr(cls,"kind");
-    int mode;
-    if (Strcmp(kind,"class") == 0) mode = PRIVATE;
-    else mode = PUBLIC;
+    int mode = PUBLIC;
+    if (kind && (Strcmp(kind,"class") == 0)) mode = PRIVATE;
     
     while (c) {
       if (Getattr(c,"error") || Getattr(c,"feature:ignore")) {
 	c = nextSibling(c);
 	continue;
       }
-      if (Strcmp(nodeType(c),"cdecl") == 0) {
+      if (!isconst && (Strcmp(nodeType(c),"extend") == 0)) {
+	methods = smart_pointer_methods(c, methods, isconst, Getattr(cls,"name"));
+      } else if (Strcmp(nodeType(c),"cdecl") == 0) {
 	if (!Getattr(c,"feature:ignore")) {
 	  String *storage = Getattr(c,"storage");
-	  if (!((Cmp(storage,"static") == 0) || (Cmp(storage,"typedef") == 0))) {
+	  if (!((Cmp(storage,"typedef") == 0))) {
 	    String *name = Getattr(c,"name");
 	    String *symname = Getattr(c,"sym:name");
 	    Node   *e    = Swig_symbol_clookup_local(name,0);
@@ -308,23 +297,30 @@ class Allocate : public Dispatcher {
 	      if (!match) {
 		Node *cc = c;
 		while (cc) {
+		  Node *cp = cc;
+		  if (classname) {
+		    Setattr(cp,"classname",classname);
+		  }
+		  Setattr(cp,"allocate:smartpointeraccess","1");
 		  /* If constant, we have to be careful */
 		  if (isconst) {
-		    SwigType *decl = Getattr(cc,"decl");
+		    SwigType *decl = Getattr(cp,"decl");
 		    if (decl) {
 		      if (SwigType_isfunction(decl)) {   /* If method, we only add if it's a const method */
 			if (SwigType_isconst(decl)) {
-			  Append(methods,cc);
+			  Append(methods,cp);
 			}
 		      } else {
-			Append(methods,cc);
+			Append(methods,cp);
 		      }
 		    } else {
-		      Append(methods,cc);
+		      Setattr(cp,"feature:immutable","1");
+		      Append(methods,cp);
 		    }
 		  } else {
-		    Append(methods,cc);
+		    Append(methods,cp);
 		  }
+		  Delete(cp);
 		  cc = Getattr(cc,"sym:nextSibling");
 		}
 	      }
@@ -371,13 +367,17 @@ class Allocate : public Dispatcher {
       }
       Node *c = Swig_symbol_clookup(t,0);
       if (c) {
-	Setattr(c,"cplus:exceptionclass","1");
+	Setattr(c,"feature:exceptionclass","1");
       }
       p = nextSibling(p);
     }
   }
 
 public:
+  Allocate() :
+    inclass(NULL),
+    extendmode(0) {}
+
   virtual int top(Node *n) {
     cplus_mode = PUBLIC;
     inclass = 0;
@@ -434,19 +434,25 @@ public:
     /* Check if the class is abstract via inheritance.   This might occur if a class didn't have
        any pure virtual methods of its own, but it didn't implement all of the pure methods in
        a base class */
-
-    if (is_abstract_inherit(n)) {
-      if ((!Getattr(n,"abstract")) && ((Getattr(n,"allocate:public_constructor") || (!Getattr(n,"feature:nodefault") && !Getattr(n,"allocate:has_constructor"))))) {
+    
+    if (!Getattr(n,"abstract") && is_abstract_inherit(n)) {
+      if (((Getattr(n,"allocate:public_constructor") || (!Getattr(n,"feature:nodefault") && !Getattr(n,"allocate:has_constructor"))))) {
 	if (!Getattr(n,"feature:notabstract")) {
 	  Node *na = Getattr(n,"abstract:firstnode");
-	  Swig_warning(WARN_TYPE_ABSTRACT, Getfile(n), Getline(n),
-		       "Class '%s' might be abstract, "
-		       "no constructors generated,\n",
-		       SwigType_namestr(Getattr(n,"name")));
-	  Swig_warning(WARN_TYPE_ABSTRACT, Getfile(na), Getline(na),
-		       " method '%s' might not be implemented.",
-		       SwigType_namestr(Getattr(na,"name")));
-	  Setattr(n,"abstract",NewList());
+	  if (na) {
+	    Swig_warning(WARN_TYPE_ABSTRACT, Getfile(n), Getline(n),
+			 "Class '%s' might be abstract, "
+			 "no constructors generated,\n",
+			 SwigType_namestr(Getattr(n,"name")));
+	    Swig_warning(WARN_TYPE_ABSTRACT, Getfile(na), Getline(na),
+			 " method '%s' might not be implemented.",
+			 SwigType_namestr(Getattr(na,"name")));
+	    if (!Getattr(n,"abstract")) {
+	      List *abstract = NewList();
+	      Append(abstract,na);
+	      Setattr(n,"abstract",abstract);
+	    }
+	  }
 	}
       }
     }
@@ -459,7 +465,7 @@ public:
       } 
       if (!Getattr(n,"allocate:default_constructor")) {
 	/* Check base classes */
-	List *bases = Getattr(n,"bases");
+	List *bases = Getattr(n,"allbases");
 	int   allows_default = 1;
 	
 	for (int i = 0; i < Len(bases); i++) {
@@ -476,7 +482,7 @@ public:
     }
     if (!Getattr(n,"allocate:has_destructor")) {
       /* No destructor was defined.  We need to check a few things here too */
-      List *bases = Getattr(n,"bases");
+      List *bases = Getattr(n,"allbases");
       int allows_destruct = 1;
 
       for (int i = 0; i < Len(bases); i++) {
@@ -491,6 +497,39 @@ public:
       }
     }
 
+    if (!Getattr(n,"allocate:has_assign")) {
+      /* No destructor was defined.  We need to check a few things here too */
+      List *bases = Getattr(n,"allbases");
+      int allows_assign = 1;
+
+      for (int i = 0; i < Len(bases); i++) {
+	Node *n = Getitem(bases,i);
+	/* If base class does not allow default destructor, we don't allow it either */
+	if (Getattr(n,"allocate:has_assign")) {
+	  allows_assign = !Getattr(n,"allocate:noassign");
+	}
+      }
+      if (!allows_assign) {
+	Setattr(n,"allocate:noassign","1");
+      }
+    }
+
+    if (!Getattr(n,"allocate:has_new")) {
+      /* No destructor was defined.  We need to check a few things here too */
+      List *bases = Getattr(n,"allbases");
+      int allows_new = 1;
+
+      for (int i = 0; i < Len(bases); i++) {
+	Node *n = Getitem(bases,i);
+	/* If base class does not allow default destructor, we don't allow it either */
+	if (Getattr(n,"allocate:has_new")) {
+	  allows_new = !Getattr(n,"allocate:nonew");
+	}
+      }
+      if (!allows_new) {
+	Setattr(n,"allocate:nonew","1");
+      }
+    }
 
     /* Check if base classes allow smart pointers, but might be hidden */
     if (!Getattr(n,"allocate:smartpointer")) {
@@ -546,11 +585,21 @@ public:
 
       String *name = Getattr(n,"name");
       if (cplus_mode != PUBLIC) {
-	/* Look for a private assignment operator */
 	if (Strcmp(name,"operator =") == 0) {
+	  /* Look for a private assignment operator */
+	  Setattr(inclass,"allocate:has_assign","1");
 	  Setattr(inclass,"allocate:noassign","1");
+	} else if (Strcmp(name,"operator new") == 0) {
+	  /* Look for a private new operator */
+	  Setattr(inclass,"allocate:has_new","1");
+	  Setattr(inclass,"allocate:nonew","1");
 	}
       } else {
+	if (Strcmp(name,"operator =") == 0) {
+	  Setattr(inclass,"allocate:has_assign","1");
+	} else if (Strcmp(name,"operator new") == 0) {
+	  Setattr(inclass,"allocate:has_new","1");
+	}
 	/* Look for smart pointer operator */
 	if ((Strcmp(name,"operator ->") == 0) && (!Getattr(n,"feature:ignore"))) {
 	  /* Look for version with no parameters */
