@@ -10,18 +10,13 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_csharp_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/csharp.cxx,v 1.44 2004/08/12 21:10:02 wsfulton Exp $";
+char cvsroot_csharp_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/csharp.cxx,v 1.49 2004/12/06 22:53:08 wsfulton Exp $";
 
 #include <limits.h> // for INT_MAX
 #include "swigmod.h"
 #include <ctype.h>
 
 
-// temporary hack start
-#define CPLUS_PUBLIC     0
-extern int cplus_mode;
-// temporary hack end
-
 class CSHARP : public Language {
   static const char *usage;
   const  String *empty_string;
@@ -54,6 +49,7 @@ class CSHARP : public Language {
   String *proxy_class_constants_code;
   String *module_class_constants_code;
   String *enum_code;
+  String *dllimport; // DllImport attribute name
   String *namespce; // Optional namespace name
   String *imclass_imports; //intermediary class imports from %pragma
   String *module_imports; //module imports from %pragma
@@ -106,6 +102,7 @@ class CSHARP : public Language {
     proxy_class_constants_code(NULL),
     module_class_constants_code(NULL),
     enum_code(NULL),
+    dllimport(NULL),
     namespce(NULL),
     imclass_imports(NULL),
     module_imports(NULL),
@@ -150,7 +147,17 @@ class CSHARP : public Language {
     // Look for certain command line options
     for (int i = 1; i < argc; i++) {
       if (argv[i]) {
-        if (strcmp(argv[i],"-namespace") == 0) {
+        if (strcmp(argv[i],"-dllimport") == 0) {
+          if (argv[i+1]) {
+            dllimport = NewString("");
+            Printf(dllimport, argv[i+1]);
+            Swig_mark_arg(i);
+            Swig_mark_arg(i+1);
+            i++;
+          } else {
+            Swig_arg_error();
+          }
+        } else if (strcmp(argv[i],"-namespace") == 0) {
           if (argv[i+1]) {
             namespce = NewString("");
             Printf(namespce, argv[i+1]);
@@ -241,13 +248,10 @@ class CSHARP : public Language {
     imclass_cppcasts_code = NewString("");
     upcasts_code = NewString("");
     if (!namespce) namespce = NewString("");
+    if (!dllimport) dllimport = Copy(module_class_name);
 
     Swig_banner(f_runtime);               // Print the SWIG banner message
 
-    if (NoInclude) {
-      Printf(f_runtime,"#define SWIG_NOINCLUDE\n");
-    }
-
     String *wrapper_name = NewString("");
 
     Printf(wrapper_name, "CSharp_%%f", imclass_name);
@@ -296,6 +300,7 @@ class CSHARP : public Language {
 
       // Add the intermediary class methods
       Replaceall(imclass_class_code, "$module", module_class_name);
+      Replaceall(imclass_class_code, "$dllimport", dllimport);
       Printv(f_im, imclass_class_code, NIL);
       Printv(f_im, imclass_cppcasts_code, NIL);
 
@@ -337,6 +342,9 @@ class CSHARP : public Language {
       Replaceall(module_class_code, "$module", module_class_name);
       Replaceall(module_class_constants_code, "$module", module_class_name);
 
+      Replaceall(module_class_code, "$dllimport", dllimport);
+      Replaceall(module_class_constants_code, "$dllimport", dllimport);
+
       // Add the wrapper methods
       Printv(f_module, module_class_code, NIL);
 
@@ -401,7 +409,7 @@ class CSHARP : public Language {
   void emitBanner(File *f) {
     Printf(f, "/* ----------------------------------------------------------------------------\n");
     Printf(f, " * This file was automatically generated by SWIG (http://www.swig.org).\n");
-    Printf(f, " * Version: %s\n", PACKAGE_VERSION);
+    Printf(f, " * Version %s\n", PACKAGE_VERSION);
     Printf(f, " *\n");
     Printf(f, " * Do not make changes to this file unless you know what you are doing--modify\n");
     Printf(f, " * the SWIG interface file instead.\n");
@@ -535,7 +543,7 @@ class CSHARP : public Language {
     }
 
     Printv(imclass_class_code, 
-           "\n  [DllImport(\"",module_class_name,"\", EntryPoint=\"CSharp_",overloaded_name,"\")]\n", NIL);
+           "\n  [DllImport(\"", dllimport, "\", EntryPoint=\"CSharp_", overloaded_name, "\")]\n", NIL);
 
     Printf(imclass_class_code, "  public static extern %s %s(", im_return_type, overloaded_name);
 
@@ -808,7 +816,7 @@ class CSHARP : public Language {
   virtual int enumDeclaration(Node *n) {
 
     if (!ImportMode) {
-      if (getCurrentClass() && (cplus_mode != CPLUS_PUBLIC)) return SWIG_NOWRAP;
+      if (getCurrentClass() && (cplus_mode != PUBLIC)) return SWIG_NOWRAP;
 
       enum_code = NewString("");
       String *symname = Getattr(n,"sym:name");
@@ -920,7 +928,7 @@ class CSHARP : public Language {
    * ---------------------------------------------------------------------- */
 
   virtual int enumvalueDeclaration(Node *n) {
-    if (getCurrentClass() && (cplus_mode != CPLUS_PUBLIC)) return SWIG_NOWRAP;
+    if (getCurrentClass() && (cplus_mode != PUBLIC)) return SWIG_NOWRAP;
 
     Swig_require("enumvalueDeclaration",n,"*name", "?value",NIL);
     String *symname = Getattr(n,"sym:name");
@@ -1027,10 +1035,9 @@ class CSHARP : public Language {
     bool const_feature_flag = const_feature && Cmp(const_feature, "0") != 0;
 
     /* Adjust the enum type for the Swig_typemap_lookup.
-     * We want the same cstype typemap for all the enum items so we use the enum type (parent node).
-     * The type of each enum item depends on what value it is assigned, but is usually a C int. */
+     * We want the same jstype typemap for all the enum items so we use the enum type (parent node). */
     if (is_enum_item) {
-      t = NewStringf("enum %s", Getattr(parentNode(n), "sym:name"));
+      t = Getattr(parentNode(n),"enumtype");
       Setattr(n,"type", t);
     }
 
@@ -1115,6 +1122,7 @@ class CSHARP : public Language {
   virtual int insertDirective(Node *n) {
     String *code = Getattr(n,"code");
     Replaceall(code, "$module", module_class_name);
+    Replaceall(code, "$dllimport", dllimport);
     return Language::insertDirective(n);
   }
 
@@ -1315,9 +1323,12 @@ class CSHARP : public Language {
     Replaceall(proxy_class_def,  "$module", module_class_name);
     Replaceall(proxy_class_code, "$module", module_class_name);
 
+    Replaceall(proxy_class_def,  "$dllimport", dllimport);
+    Replaceall(proxy_class_code, "$dllimport", dllimport);
+
     // Add code to do C++ casting to base class (only for classes in an inheritance hierarchy)
     if(derived){
-      Printv(imclass_cppcasts_code,"\n  [DllImport(\"", module_class_name, "\", EntryPoint=\"CSharp_", proxy_class_name ,"Upcast", "\")]\n", NIL);
+      Printv(imclass_cppcasts_code,"\n  [DllImport(\"", dllimport, "\", EntryPoint=\"CSharp_", proxy_class_name ,"Upcast", "\")]\n", NIL);
       Printf(imclass_cppcasts_code,"  public static extern IntPtr $csclassnameUpcast(IntPtr objectRef);\n");
 
       Replaceall(imclass_cppcasts_code, "$csclassname", proxy_class_name);
@@ -1390,6 +1401,11 @@ class CSHARP : public Language {
       Replaceall(proxy_class_def, "$module", module_class_name);
       Replaceall(proxy_class_code, "$module", module_class_name);
       Replaceall(proxy_class_constants_code, "$module", module_class_name);
+
+      Replaceall(proxy_class_def, "$dllimport", dllimport);
+      Replaceall(proxy_class_code, "$dllimport", dllimport);
+      Replaceall(proxy_class_constants_code, "$dllimport", dllimport);
+
       Printv(f_proxy, proxy_class_def, proxy_class_code, NIL);
 
       // Write out all the constants
@@ -2151,47 +2167,63 @@ class CSHARP : public Language {
 
   bool substituteClassname(SwigType *pt, String *tm) {
     bool substitution_performed = false;
-    if (Strstr(tm, "$csclassname") || Strstr(tm,"$&csclassname")) {
-      SwigType *type = Copy(SwigType_typedef_resolve_all(pt));
-      SwigType *strippedtype = SwigType_strip_qualifiers(type);
+    SwigType *type = Copy(SwigType_typedef_resolve_all(pt));
+    SwigType *strippedtype = SwigType_strip_qualifiers(type);
 
-      if (SwigType_isenum(strippedtype)) {
-        String *enumname = getEnumName(pt);
-        if (enumname)
-          Replaceall(tm, "$csclassname", enumname);
-        else
-          Replaceall(tm, "$csclassname", NewStringf("int"));
-      } else {
-        String *classname = getProxyName(pt);
-        if (classname) {
-          Replaceall(tm,"$&csclassname", classname); // getProxyName() works for pointers to classes too
-          Replaceall(tm,"$csclassname", classname);
-        }
-        else { // use $descriptor if SWIG does not know anything about this type. Note that any typedefs are resolved.
-          String *descriptor = NULL;
-
-          if (Strstr(tm, "$&csclassname")) {
-            SwigType_add_pointer(type);
-            descriptor = NewStringf("SWIGTYPE%s", SwigType_manglestr(type));
-            Replaceall(tm, "$&csclassname", descriptor);
-          }
-          else { // $csclassname
-            descriptor = NewStringf("SWIGTYPE%s", SwigType_manglestr(type));
-            Replaceall(tm, "$csclassname", descriptor);
-          }
-
-          // Add to hash table so that the type wrapper classes can be created later
-          Setattr(swig_types_hash, descriptor, type);
-          Delete(descriptor);
-        }
-      }
+    if (Strstr(tm, "$csclassname")) {
+      SwigType *classnametype = Copy(strippedtype);
+      substituteClassnameSpecialVariable(classnametype, tm, "$csclassname");
       substitution_performed = true;
-      Delete(type);
-      Delete(strippedtype);
+      Delete(classnametype);
     }
+    if (Strstr(tm, "$*csclassname")) {
+      SwigType *classnametype = Copy(strippedtype);
+      Delete(SwigType_pop(classnametype));
+      substituteClassnameSpecialVariable(classnametype, tm, "$*csclassname");
+      substitution_performed = true;
+      Delete(classnametype);
+    }
+    if (Strstr(tm, "$&csclassname")) {
+      SwigType *classnametype = Copy(strippedtype);
+      SwigType_add_pointer(classnametype);
+      substituteClassnameSpecialVariable(classnametype, tm, "$&csclassname");
+      substitution_performed = true;
+      Delete(classnametype);
+    }
+
+    Delete(strippedtype);
+    Delete(type);
+
     return substitution_performed;
   }
 
+  /* -----------------------------------------------------------------------------
+   * substituteClassnameSpecialVariable()
+   * ----------------------------------------------------------------------------- */
+
+  void substituteClassnameSpecialVariable(SwigType *classnametype, String *tm, const char *classnamespecialvariable) {
+    if (SwigType_isenum(classnametype)) {
+      String *enumname = getEnumName(classnametype);
+      if (enumname)
+        Replaceall(tm, classnamespecialvariable, enumname);
+      else
+        Replaceall(tm, classnamespecialvariable, NewStringf("int"));
+    } else {
+      String *classname = getProxyName(classnametype);
+      if (classname) {
+        Replaceall(tm, classnamespecialvariable, classname); // getProxyName() works for pointers to classes too
+      }
+      else { // use $descriptor if SWIG does not know anything about this type. Note that any typedefs are resolved.
+        String *descriptor = NewStringf("SWIGTYPE%s", SwigType_manglestr(classnametype));
+        Replaceall(tm, classnamespecialvariable, descriptor);
+
+        // Add to hash table so that the type wrapper classes can be created later
+        Setattr(swig_types_hash, descriptor, classnametype);
+        Delete(descriptor);
+      }
+    }
+  }
+
   /* -----------------------------------------------------------------------------
    * makeParameterName()
    *
@@ -2268,6 +2300,7 @@ class CSHARP : public Language {
 
     Replaceall(swigtype, "$csclassname", classname);
     Replaceall(swigtype, "$module", module_class_name);
+    Replaceall(swigtype, "$dllimport", dllimport);
     Printv(f_swigtype, swigtype, NIL);
 
     Close(f_swigtype);
@@ -2378,6 +2411,7 @@ extern "C" Language * swig_csharp(void) {
 
 const char *CSHARP::usage = (char*)"\
 C# Options (available with -csharp)\n\
+     -dllimport <dl> - Override DllImport attribute name to <dl>\n\
      -namespace <nm> - Generate wrappers into C# namespace <nm>\n\
      -noproxy        - Generate the low-level functional interface instead\n\
                        of proxy classes\n\
