@@ -13,10 +13,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-char cvsroot_guile_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.22 2004/09/02 20:32:30 wsfulton Exp $";
+char cvsroot_guile_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.25 2004/11/04 01:02:21 wuzzeb Exp $";
 
 /***********************************************************************
- * $Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.22 2004/09/02 20:32:30 wsfulton Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Modules/guile.cxx,v 1.25 2004/11/04 01:02:21 wuzzeb Exp $
  *
  * guile.cxx
  *
@@ -55,8 +55,8 @@ Guile Options (available with -guile)\n\
                                modules.\n\
      -scmstub                - Output Scheme file with module declaration and\n\
                                exports; only with `passive' and `simple' linkage\n\
-     -gh                     - Use the gh_ Guile API. (Guile <= 1.8, default) \n\
-     -scm                    - Use the scm Guile API. (Guile >= 1.6) \n\
+     -gh                     - Use the gh_ Guile API. (Guile <= 1.8) \n\
+     -scm                    - Use the scm Guile API. (Guile >= 1.6, default) \n\
      -proxy                  - Export GOOPS class definitions\n\
      -emitslotaccessors      - Emit accessor methods for all GOOPS slots\n" "\
      -primsuffix <suffix>    - Name appended to primitive module when exporting\n\
@@ -109,7 +109,7 @@ static String *return_multi_doc = 0;
 
 static String *exported_symbols = 0;
 
-static int     use_scm_interface = 0;
+static int     use_scm_interface = 1;
 static int     exporting_destructor = 0;
 static String *swigtype_ptr = 0;
 
@@ -356,13 +356,16 @@ public:
     Swig_banner (f_runtime);
     
     Printf (f_runtime, "/* Implementation : GUILE */\n\n");
+
+    if (!use_scm_interface) {
+      if (SwigRuntime == 1)
+	Printf(f_runtime, "#define SWIG_GLOBAL\n");
+      if (SwigRuntime == 2)
+        Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
+    }
     
     /* Write out directives and declarations */
     
-    if (NoInclude) {
-      Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
-    }
-    
     module = Swig_copy_string(Char(Getattr(n,"name")));
     
     if (CPlusPlus) {
