@@ -13,10 +13,10 @@
  * can be used and distributed.
  *****************************************************************************/
 
-char cvsroot_mzscheme_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.10 2004/08/17 14:20:13 mkoeppe Exp $";
+char cvsroot_mzscheme_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.11 2004/10/05 22:43:41 wuzzeb Exp $";
 
 /***********************************************************************
- * $Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.10 2004/08/17 14:20:13 mkoeppe Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Modules/mzscheme.cxx,v 1.11 2004/10/05 22:43:41 wuzzeb Exp $
  *
  * mzscheme.cxx
  *
@@ -149,10 +149,6 @@ public:
     Printf(f_runtime, "/* -*- buffer-read-only: t -*- vi: set ro: */\n");
     Swig_banner (f_runtime);
     
-    if (NoInclude) {
-      Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
-    }
-    
     module = Getattr(n,"name");
     
     Language::top(n);
@@ -583,7 +579,7 @@ public:
     
     // Make a static variable;
     
-    Printf (var_name, "_wrap_const_%s", Swig_name_mangle(iname));
+    Printf (var_name, "_wrap_const_%s", Swig_name_mangle(Getattr(n,"sym:name")));
     
     // Build the name for scheme.
     Printv(proc_name, iname,NIL);
