@@ -13,7 +13,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ------------------------------------------------------------------------- */
 
-char cvsroot_perl5_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/perl5.cxx,v 1.15 2004/09/02 20:33:28 wsfulton Exp $";
+char cvsroot_perl5_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/perl5.cxx,v 1.17 2004/12/01 00:50:05 marcelomatus Exp $";
 
 #include "swigmod.h"
 
@@ -214,10 +214,6 @@ public:
 
     Swig_banner(f_runtime);
   
-    if (NoInclude) {
-      Printf(f_runtime,"#define SWIG_NOINCLUDE\n");
-    }
-
     module = Copy(Getattr(n,"name"));
 
   /* If we're in blessed mode, change the package name to "packagec" */
@@ -1075,10 +1071,13 @@ public:
    * ------------------------------------------------------------ */
   virtual int classDeclaration(Node *n) {
     /* Do some work on the class name */
-    if (blessed) {
-      setclassname(n);
-      Append(classlist,n);
+    if (!Getattr(n,"feature:onlychildren")) {
+      if (blessed) {
+	setclassname(n);
+	Append(classlist,n);
+      }
     }
+    
     return Language::classDeclaration(n);
   }
   
