@@ -18,7 +18,7 @@
  *
  ***********************************************************************/
  
-char cvsroot_pike_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/pike.cxx,v 1.12 2004/07/10 16:42:54 wsfulton Exp $";
+char cvsroot_pike_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/pike.cxx,v 1.13 2004/10/16 18:23:59 wuzzeb Exp $";
 
 #include "swigmod.h"
 
@@ -135,9 +135,6 @@ public:
 
     /* Standard stuff for the SWIG runtime section */    
     Swig_banner(f_runtime);
-    if (NoInclude) {
-      Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
-    }
 
     Printf(f_header, "#define SWIG_init    pike_module_init\n");
     Printf(f_header, "#define SWIG_name    \"%s\"\n\n", module);
