@@ -1,7 +1,7 @@
 /********************************************************************
  * Ruby module for SWIG
  *
- * $Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.58 2004/09/01 02:49:23 lyle Exp $
+ * $Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.71 2004/12/13 22:41:13 wsfulton Exp $
  *
  * Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
  * Copyright (C) 2000  Information-technology Promotion Agency, Japan
@@ -10,7 +10,7 @@
  *
  ********************************************************************/
 
-char cvsroot_ruby_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.58 2004/09/01 02:49:23 lyle Exp $";
+char cvsroot_ruby_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/ruby.cxx,v 1.71 2004/12/13 22:41:13 wsfulton Exp $";
 
 #include "swigmod.h"
 #define SWIG_PROTECTED_TARGET_METHODS 1
@@ -148,7 +148,6 @@ private:
   File *f_wrappers;
   File *f_init;
 
-  bool use_kw;
   bool useGlobalModule;
   bool multipleInheritance;
 
@@ -186,9 +185,18 @@ public:
     f_header = 0;
     f_wrappers = 0;
     f_init = 0;
-    use_kw = false;
     useGlobalModule = false;
     multipleInheritance = false;
+    director_prot_ctor_code = NewString("");    
+    Printv(director_prot_ctor_code,
+	   "if ( $comparison ) { /* subclassed */\n",
+	   "  $director_new \n",
+	   "} else {\n",
+	   "  rb_raise(rb_eRuntimeError,\"accessing abstract class or protected constructor\"); \n",
+	   "  return Qnil;\n",
+	   "}\n", NIL);
+    director_multiple_inheritance = 0;
+    director_language = 1;
   }
   
   /* ---------------------------------------------------------------------
@@ -220,6 +228,7 @@ public:
 	  Swig_mark_arg(i);
 	} else if (strcmp(argv[i],"-minherit") == 0) {
           multipleInheritance = true;
+	  director_multiple_inheritance = 1;
 	  Swig_mark_arg(i);
         } else if (strcmp(argv[i],"-prefix") == 0) {
           if (argv[i+1]) {
@@ -348,12 +357,15 @@ public:
         }
         if (Getattr(options, "ruby_minherit")) {
           multipleInheritance = true;
+	  director_multiple_inheritance = 1;
         }
       }
     }
 
     /* Set comparison with none for ConstructorToFunction */
-    setSubclassInstanceCheck(NewString("CLASS_OF(self) != Qnil")); // FIXME
+
+    
+    setSubclassInstanceCheck(NewStringf("strcmp(rb_obj_classname(self), classname) != 0")); 
     // setSubclassInstanceCheck(NewString("CLASS_OF(self) != cFoo.klass"));
 
     /* Initialize all of the output files */
@@ -397,10 +409,6 @@ public:
 
     Swig_banner(f_runtime);
 
-    if (NoInclude) {
-      Printf(f_runtime, "#define SWIG_NOINCLUDE\n");
-    }
-
     if (directorsEnabled()) {
       Printf(f_runtime,"#define SWIG_DIRECTORS\n");
     }
@@ -628,15 +636,15 @@ public:
     String *s = NewString("");
     String *temp = NewString("");
     
+#ifdef SWIG_PROTECTED_TARGET_METHODS
+    const char* rb_define_method = is_public(n) ?
+      "rb_define_method" : "rb_define_protected_method" ;
+#else        
+    const char* rb_define_method = "rb_define_method";
+#endif	
     switch (current) {
     case MEMBER_FUNC: 
       {
-#ifdef SWIG_PROTECTED_TARGET_METHODS
-	const char* rb_define_method = is_public(n) ?
-	  "rb_define_method" : "rb_define_protected_method" ;
-#else        
-	const char* rb_define_method = "rb_define_method";
-#endif	
 	if (multipleInheritance) {
 	  Printv(klass->init, tab4, rb_define_method,"(", klass->mImpl, ", \"",
 		 iname, "\", ", wname, ", -1);\n", NIL);
@@ -651,7 +659,7 @@ public:
       Replaceall(klass->init,"$allocator", s);
       break;
     case CONSTRUCTOR_INITIALIZE:
-      Printv(s, tab4, "rb_define_method(", klass->vname,
+      Printv(s, tab4, rb_define_method,"(", klass->vname,
 	     ", \"initialize\", ", wname, ", -1);\n", NIL);
       Replaceall(klass->init,"$initializer", s);
       break;
@@ -749,7 +757,7 @@ public:
     source = NewString("");
     target = NewString("");
 
-    bool use_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
+    bool ctor_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
 
     /**
      * The 'start' value indicates which of the C/C++ function arguments
@@ -758,7 +766,7 @@ public:
      * the first argument (with name arg1) is based on the value of argv[0].
      * If start is one, then arg1 is based on the value of argv[1].
      */
-    int start = (current == MEMBER_FUNC || current == MEMBER_VAR || use_director) ? 1 : 0;
+    int start = (current == MEMBER_FUNC || current == MEMBER_VAR || ctor_director) ? 1 : 0;
 
     int varargs = emit_isvarargs(l);
 
@@ -1000,10 +1008,10 @@ public:
     int  numarg = emit_num_arguments(l);
     int  numreq = emit_num_required(l);
     int  varargs = emit_isvarargs(l);
-    bool allow_kwargs = use_kw || Getattr(n,"feature:kwargs");
+    bool allow_kwargs = Getattr(n,"feature:kwargs") ? true : false;
     
-    bool use_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
-    int start = (current == MEMBER_FUNC || current == MEMBER_VAR || use_director) ? 1 : 0;
+    bool ctor_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
+    int start = (current == MEMBER_FUNC || current == MEMBER_VAR || ctor_director) ? 1 : 0;
 
     /* Now write the wrapper function itself */
     if        (current == CONSTRUCTOR_ALLOCATE) {
@@ -1037,7 +1045,7 @@ public:
     }
 
     // FIXME?
-    if (use_director) {
+    if (ctor_director) {
       numarg--;
       numreq--;
     }
@@ -1073,7 +1081,8 @@ public:
 
     if (directorsEnabled()) {
       if (!is_smart_pointer()) {
-        if (/*directorbase &&*/ !constructor && !destructor && isVirtual) {
+        if (/*directorbase &&*/ !constructor && !destructor 
+	    && isVirtual  && !Getattr(n,"feature:nodirector")) {
           Wrapper_add_local(f, "director", "Swig::Director *director = 0");
           Printf(f->code, "director = dynamic_cast<Swig::Director *>(arg1);\n");
           Printf(f->code, "if (director && (director->swig_get_self() == self)) director->swig_set_up();\n");
@@ -1250,14 +1259,16 @@ public:
 	   NIL);
     
     Wrapper_add_local(f, "argc", "int argc");
-    if (current == MEMBER_FUNC || current == MEMBER_VAR) {
+    bool ctor_director = (current == CONSTRUCTOR_INITIALIZE && Swig_directorclass(n));
+    if (current == MEMBER_FUNC || current == MEMBER_VAR || ctor_director) {
       Printf(tmp, "VALUE argv[%d]", maxargs+1);
     } else {
       Printf(tmp, "VALUE argv[%d]", maxargs);
     }
     Wrapper_add_local(f, "argv", tmp);
     Wrapper_add_local(f, "ii", "int ii");
-    if (current == MEMBER_FUNC || current == MEMBER_VAR) {
+
+    if (current == MEMBER_FUNC || current == MEMBER_VAR || ctor_director) {
       Printf(f->code, "argc = nargs + 1;\n");
       Printf(f->code, "argv[0] = self;\n");
       Printf(f->code, "for (ii = 1; (ii < argc) && (ii < %d); ii++) {\n", maxargs);
@@ -1428,7 +1439,7 @@ public:
     }
     
     Swig_warning(WARN_RUBY_WRONG_NAME, input_file, line_number,
-		 "Wrong %s name\n", reason);
+		 "Wrong %s name %s\n", reason, name);
     
     return name;
   }
@@ -1494,22 +1505,24 @@ public:
    * ----------------------------------------------------------------------------- */
 
   virtual int classDeclaration(Node *n) {
-    String *name = Getattr(n,"name");
-    String *symname = Getattr(n,"sym:name");
-    String *tdname = Getattr(n,"tdname");
-  
-    name = tdname ? tdname : name;
-    String *namestr = SwigType_namestr(name);
-    klass = RCLASS(classes, Char(namestr));
-    if (!klass) {
-      klass = new RClass();
-      String *valid_name = NewString(symname ? symname : namestr);
-      validate_const_name(Char(valid_name), "class");
-      klass->set_name(namestr, symname, valid_name);
-      SET_RCLASS(classes, Char(namestr), klass);
-      Delete(valid_name);
-    }
-    Delete(namestr);
+    if (!Getattr(n,"feature:onlychildren")) {
+      String *name = Getattr(n,"name");
+      String *symname = Getattr(n,"sym:name");
+      String *tdname = Getattr(n,"tdname");
+    
+      name = tdname ? tdname : name;
+      String *namestr = SwigType_namestr(name);
+      klass = RCLASS(classes, Char(namestr));
+      if (!klass) {
+	klass = new RClass();
+	String *valid_name = NewString(symname ? symname : namestr);
+	validate_const_name(Char(valid_name), "class");
+	klass->set_name(namestr, symname, valid_name);
+	SET_RCLASS(classes, Char(namestr), klass);
+	Delete(valid_name);
+      }
+      Delete(namestr);
+    }    
     return Language::classDeclaration(n);
   }
 
@@ -1713,12 +1726,47 @@ public:
    * Method for adding C++ member constructor
    * -------------------------------------------------------------------- */
 
+  void set_director_ctor_code(Node *n) 
+  {
+    /* director ctor code is specific for each class */
+    Delete(director_prot_ctor_code);
+    director_prot_ctor_code = NewString("");
+    Node *pn = Swig_methodclass(n);
+    String *symname = Getattr(pn,"sym:name");
+    String  *name   = Copy(symname);
+    char *cname = Char(name);
+    if(cname) cname[0] = toupper(cname[0]);
+    Printv(director_prot_ctor_code,
+	   "char *classname = \"",module,"::",cname,"\";\n",
+	   "if ( $comparison ) { /* subclassed */\n",
+	   "  $director_new \n",
+	   "} else {\n",
+	   "  rb_raise(rb_eNameError,\"accessing abstract class or protected constructor\"); \n",
+	   "  return Qnil;\n",
+	   "}\n", NIL);
+    Delete(director_ctor_code);
+    director_ctor_code = NewString("");
+    Printv(director_ctor_code,
+	   "char *classname = \"",module,"::",cname,"\";\n",
+	   "if ( $comparison ) { /* subclassed */\n",
+	   "  $director_new \n",
+	   "} else {\n",
+	   "  $nondirector_new \n",
+	   "}\n", NIL);
+    Delete(name);
+  }
+
   virtual int constructorHandler(Node *n) {
     int   use_director = Swig_directorclass(n);
-
+    if (use_director) {
+      set_director_ctor_code(n);
+    }
+    
     /* First wrap the allocate method */
     current = CONSTRUCTOR_ALLOCATE;
     Swig_name_register((String_or_char *) "construct", (String_or_char *) "%c_allocate");
+    
+    
     Language::constructorHandler(n);
 
     /* 
@@ -1758,6 +1806,20 @@ public:
     return SWIG_OK;
   }
 
+  virtual int copyconstructorHandler(Node *n) {
+    int   use_director = Swig_directorclass(n);
+    if (use_director) {
+      set_director_ctor_code(n);
+    }
+
+    /* First wrap the allocate method */
+    current = CONSTRUCTOR_ALLOCATE;
+    Swig_name_register((String_or_char *) "construct", (String_or_char *) "%c_allocate");
+    
+    return Language::copyconstructorHandler(n);
+  }
+  
+
   /* ---------------------------------------------------------------------
    * destructorHandler()
    * -------------------------------------------------------------------- */
@@ -1880,10 +1942,6 @@ public:
     return Language::classDirectorEnd(n);
   }
   
-  virtual int unrollVirtualMethods(Node *n, Node *parent, Hash *vm, int default_director, int &virtual_destructor) {
-    return Language::unrollVirtualMethods(n, parent, vm, default_director, virtual_destructor);
-  }
-  
   /* ------------------------------------------------------------
    * classDirectorConstructor()
    * ------------------------------------------------------------ */
@@ -1896,40 +1954,36 @@ public:
     String *classname = NewString("");
     Printf(classname, "SwigDirector_%s", supername);
 
-    /* insert self and disown parameters */
-    Parm *p, *ip;
+    /* insert self parameter */
+    Parm *p;
     ParmList *superparms = Getattr(n, "parms");
     ParmList *parms = CopyParmList(superparms);
     String *type = NewString("VALUE");
     p = NewParm(type, NewString("self"));
     set_nextSibling(p, parms);
     parms = p;
-    for (ip = parms; nextSibling(ip); ) ip = nextSibling(ip);
-    p = NewParm(NewString("bool"), NewString("disown"));
-    Setattr(p, "arg:byname", "1");
-    Setattr(n, "director:postfix_args", p);
-    Setattr(p, "value", "false");
-    set_nextSibling(ip, p);
     
-    /* constructor */
-    {
-      Wrapper *w = NewWrapper();
-      String *call;
-      String *basetype = Getattr(parent, "classtype");
-      String *target = Swig_method_decl(decl, classname, parms, 0, 0);
-      call = Swig_csuperclass_call(0, basetype, superparms);
-      Printf(w->def, "%s::%s: %s, Swig::Director(self, disown) { }", classname, target, call);
-      Delete(target);
-      Wrapper_print(w, f_directors);
-      Delete(call);
-      DelWrapper(w);
-    }
-    
-    /* constructor header */
-    {
-      String *target = Swig_method_decl(decl, classname, parms, 0, 1);
-      Printf(f_directors_h, "    %s;\n", target);
-      Delete(target);
+    if (!Getattr(n,"defaultargs")) {
+      /* constructor */
+      {
+        Wrapper *w = NewWrapper();
+        String *call;
+        String *basetype = Getattr(parent, "classtype");
+        String *target = Swig_method_decl(decl, classname, parms, 0, 0);
+        call = Swig_csuperclass_call(0, basetype, superparms);
+        Printf(w->def, "%s::%s: %s, Swig::Director(self) { }", classname, target, call);
+        Delete(target);
+        Wrapper_print(w, f_directors);
+        Delete(call);
+        DelWrapper(w);
+      }
+      
+      /* constructor header */
+      {
+        String *target = Swig_method_decl(decl, classname, parms, 0, 1);
+        Printf(f_directors_h, "    %s;\n", target);
+        Delete(target);
+      }
     }
 
     Delete(sub);
@@ -1939,21 +1993,25 @@ public:
     return Language::classDirectorConstructor(n);
   }
   
+  /* ------------------------------------------------------------
+   * classDirectorDefaultConstructor()
+   * ------------------------------------------------------------ */
+
   virtual int classDirectorDefaultConstructor(Node *n) {
     String *classname;
     Wrapper *w;
     classname = Swig_class_name(n);
     w = NewWrapper();
-    Printf(w->def, "SwigDirector_%s::SwigDirector_%s(VALUE self, bool disown) : Swig::Director(self, disown) { }", classname, classname);
+    Printf(w->def, "SwigDirector_%s::SwigDirector_%s(VALUE self) : Swig::Director(self) { }", classname, classname);
     Wrapper_print(w, f_directors);
     DelWrapper(w);
-    Printf(f_directors_h, "    SwigDirector_%s(VALUE self, bool disown = true);\n", classname);
+    Printf(f_directors_h, "    SwigDirector_%s(VALUE self);\n", classname);
     Delete(classname);
     return Language::classDirectorDefaultConstructor(n);
   }
   
   /* ---------------------------------------------------------------
-   * classDirectorMethod()
+   * exceptionSafeMethodCall()
    *
    * Emit a virtual director method to pass a method call on to the 
    * underlying Ruby instance.
@@ -2009,9 +2067,9 @@ public:
       if (argc > 0) {
         Wrapper_add_localv(w, "i", "int", "i", NIL);
         Printf(w->code, "args.argv = new VALUE[%d];\n", argc);
-        Printf(w->code, "for (i = 0; i < %d; i++) {\n", argc);
-        Printv(w->code, "args.argv[i] = Qnil;\n", NIL);
-        Printv(w->code, "}\n", NIL);
+        for (int i = 0; i < argc; i++) {
+          Printf(w->code, "args.argv[%d] = obj%d;\n", i, i);
+        } 
       } else {
         Printv(w->code, "args.argv = 0;\n", NIL);
       }
@@ -2116,14 +2174,14 @@ public:
     // Get any exception classes in the throws typemap
     ParmList *throw_parm_list = 0;
 
-    if ((throw_parm_list = Getattr(n,"throws"))) {
+    if ((throw_parm_list = Getattr(n,"throws")) || Getattr(n,"throw")) {
       Parm      *p;
       int       gencomma = 0;
 
       Append(w->def, " throw(");
       Append(declaration, " throw(");
 
-      Swig_typemap_attach_parms("throws", throw_parm_list, 0);
+      if (throw_parm_list) Swig_typemap_attach_parms("throws", throw_parm_list, 0);
       for (p = throw_parm_list; p; p=nextSibling(p)) {
         if ((tm = Getattr(p,"tmap:throws"))) {
           if (gencomma++) {
@@ -2366,19 +2424,23 @@ public:
 
     /* any existing helper functions to handle this? */
     if (!is_void) {
+      String* rettype = SwigType_str(return_type, 0);      
       if (!SwigType_isreference(return_type)) {
-        Printf(w->code, "return c_result;\n");
+	Printf(w->code, "return (%s) c_result;\n", rettype);
       } else {
-        Printf(w->code, "return *c_result;\n");
+	Printf(w->code, "return (%s) *c_result;\n", rettype);
       }
+      Delete(rettype);
     }
 
     Printf(w->code, "}\n");
 
     /* emit the director method */
     if (status == SWIG_OK) {
-      Wrapper_print(w, f_directors);
-      Printv(f_directors_h, declaration, NIL);
+      if (!Getattr(n,"defaultargs")) {
+        Wrapper_print(w, f_directors);
+        Printv(f_directors_h, declaration, NIL);
+      }
     }
 
     /* clean up */
