@@ -20,7 +20,7 @@
  *
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_tcl8_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/tcl8.cxx,v 1.12 2004/08/23 20:05:51 wsfulton Exp $";
+char cvsroot_tcl8_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/tcl8.cxx,v 1.15 2004/10/29 22:28:00 marcelomatus Exp $";
 
 #include "swigmod.h"
 
@@ -43,6 +43,7 @@ static int         nspace = 0;
 static String     *init_name = 0;
 static String     *ns_name = 0;
 static int         have_constructor;
+static String     *constructor_name;
 static int         have_destructor;
 static int         have_base_classes;
 static String     *destructor_action = 0;
@@ -157,11 +158,6 @@ public:
     
     Swig_banner(f_runtime);
     
-    /* Include a Tcl configuration file */
-    if (NoInclude) {
-      Printf(f_runtime,"#define SWIG_NOINCLUDE\n");
-    }
-    
     /* Set the module name, namespace, and prefix */
     
     module = NewStringf("%(lower)s", Getattr(n,"name"));
@@ -618,6 +614,7 @@ public:
   virtual int constantWrapper(Node *n) {
     String *name      = Getattr(n,"name");
     String *iname     = Getattr(n,"sym:name");
+    String *nsname    = !nspace ? Copy(iname) : NewStringf("%s::%s",ns_name,iname);
     SwigType *type    = Getattr(n,"type");
     String   *value   = Getattr(n,"value");
     String *tm;
@@ -630,21 +627,26 @@ public:
       Printf(f_wrappers, "static %s = %s;\n", SwigType_str(type,wname), value);
       value = Char(wname);
     }
+    
     if ((tm = Swig_typemap_lookup_new("consttab",n,name,0))) {
       Replaceall(tm,"$source",value);
       Replaceall(tm,"$target",name);
       Replaceall(tm,"$value",value);
+      Replaceall(tm,"$nsname",nsname);
       Printf(const_tab,"%s,\n", tm);
     } else if ((tm = Swig_typemap_lookup_new("constcode", n, name, 0))) {
       Replaceall(tm,"$source", value);
       Replaceall(tm,"$target", name);
       Replaceall(tm,"$value",value);
+      Replaceall(tm,"$nsname",nsname);
       Printf(f_init, "%s\n", tm);
     } else {
+      Delete(nsname);
       Swig_warning(WARN_TYPEMAP_CONST_UNDEF,
 		   input_file, line_number, "Unsupported constant value.\n");
       return SWIG_NOWRAP;
     }
+    Delete(nsname);
     return SWIG_OK;
   }
 
@@ -747,6 +749,7 @@ public:
     /* Handle inheritance */
   
     String *base_class = NewString("");
+    String *base_class_names = NewString("");
 
     if( itcl ) {
       base_classes    = NewString("");
@@ -772,10 +775,11 @@ public:
 	//	Printv(f_wrappers,"extern swig_class _wrap_class_", bmangle, ";\n", NIL);
 	//	Printf(base_class,"&_wrap_class_%s",bmangle);
 	Printf(base_class,"0");
+	Printf(base_class_names,"\"%s *\",", SwigType_namestr(bname));
 	/* Put code to register base classes in init function */
 	
-	Printf(f_init,"/* Register base : %s */\n", bmangle);
-	Printf(f_init,"swig_%s_bases[%d] = (swig_class *) SWIG_TypeQuery(\"%s *\")->clientdata;\n",  mangled_classname, index, SwigType_namestr(bname));
+	//Printf(f_init,"/* Register base : %s */\n", bmangle);
+	//Printf(f_init,"swig_%s_bases[%d] = (swig_class *) SWIG_TypeQuery(\"%s *\")->clientdata;\n",  mangled_classname, index, SwigType_namestr(bname));
 	b = Next(b);
 	index++;
 	Putc(',',base_class);
@@ -885,13 +889,17 @@ public:
     };
 
     Printv(f_wrappers,"static swig_class *swig_",mangled_classname,"_bases[] = {", base_class,"0};\n", NIL);
+    Printv(f_wrappers,"static char *swig_",mangled_classname,"_base_names[] = {", base_class_names,"0};\n", NIL);
     Delete(base_class);
+    Delete(base_class_names);
 
     Printv(f_wrappers, "swig_class _wrap_class_", mangled_classname, " = { \"", class_name,
 	   "\", &SWIGTYPE", SwigType_manglestr(t), ",",NIL);
   
     if (have_constructor) {
-      Printf(f_wrappers,"%s", Swig_name_wrapper(Swig_name_construct(class_name)));
+      Printf(f_wrappers,"%s", Swig_name_wrapper(Swig_name_construct(constructor_name)));
+      Delete(constructor_name);
+      constructor_name = 0;
     } else {
       Printf(f_wrappers,"0");
     }
@@ -900,7 +908,8 @@ public:
     } else {
       Printf(f_wrappers,",0");
     }
-    Printv(f_wrappers, ", swig_", mangled_classname, "_methods, swig_", mangled_classname, "_attributes, swig_", mangled_classname,"_bases };\n", NIL);
+    Printv(f_wrappers, ", swig_", mangled_classname, "_methods, swig_", mangled_classname, "_attributes, swig_", mangled_classname,"_bases,",
+		         "swig_", mangled_classname, "_base_names };\n", NIL);
 
     if( !itcl ) {
       Printv(cmd_tab, tab4, "{ SWIG_prefix \"", class_name, "\", (swig_wrapper_func) SWIG_ObjectConstructor, &_wrap_class_", mangled_classname, "},\n", NIL);
@@ -1141,6 +1150,7 @@ public:
       }
     }
 
+    constructor_name = NewString(Getattr(n, "sym:name"));
     have_constructor = 1;
     return SWIG_OK;
   }
