@@ -15,7 +15,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cpp_c[] = "$Header: /cvsroot/swig/SWIG/Source/Preprocessor/cpp.c,v 1.53 2004/03/17 08:32:11 mmatus Exp $";
+char cvsroot_cpp_c[] = "$Header: /cvsroot/swig/SWIG/Source/Preprocessor/cpp.c,v 1.62 2004/11/18 00:25:23 marcelomatus Exp $";
 
 #include "swig.h"
 #include "preprocessor.h"
@@ -25,6 +25,7 @@ static Hash     *cpp = 0;                /* C preprocessor data */
 static int       include_all = 0;        /* Follow all includes */
 static int       ignore_missing = 0; 
 static int       import_all = 0;         /* Follow all includes, but as %import statements */
+static int       imported_depth = 0;	 /* Depth of %imported files */
 static int       single_include = 1;     /* Only include each file once */
 static Hash     *included_files = 0;
 static List     *dependencies = 0;
@@ -77,9 +78,9 @@ copy_location(DOH *s1, DOH *s2) {
   Setline(s2,Getline(s1));
 }
 
-static String *cpp_include(String_or_char *fn) {
+static String *cpp_include(String_or_char *fn, int sysfile) {
   String *s;
-  s = Swig_include(fn);
+  s = sysfile ? Swig_include_sys(fn) : Swig_include(fn);
   if (s && single_include) {
     String *file = Getfile(s);
     if (Getattr(included_files,file)) {
@@ -484,7 +485,7 @@ find_args(String *s)
  * ----------------------------------------------------------------------------- */
 
 static String *
-get_filename(String *str) {
+get_filename(String *str, int* sysfile) {
   String *fn;
   int  c;
 
@@ -492,9 +493,11 @@ get_filename(String *str) {
   fn = NewString("");
   copy_location(str,fn);
   c = Getc(str);
+  *sysfile = 0;
   if (c == '\"') {
     while (((c = Getc(str)) != EOF) && (c != '\"')) Putc(c,fn);
   } else if (c == '<') {
+    *sysfile = 1;
     while (((c = Getc(str)) != EOF) && (c != '>'))  Putc(c,fn);
   } else {
     Putc(c,fn);
@@ -615,7 +618,7 @@ expand_macro(String_or_char *name, List *args)
   }
 
   /* If the macro expects arguments, but none were supplied, we leave it in place */
-  if (!args && (margs)) {
+  if (!args && (margs) && Len(margs) > 0) {
     return NewString(name);
   }
 
@@ -831,30 +834,32 @@ Preprocessor_replace(DOH *s)
 	Ungetc(c,s);
 	/* See if this is the special "defined" macro */
 	if (Cmp(id,"defined") == 0) {
-	  DOH *args;
+	  DOH *args = 0;
 	  /* See whether or not a paranthesis has been used */
 	  skip_whitespace(s,0);
 	  c = Getc(s);
 	  if (c == '(') {
 	    Seek(s,-1,SEEK_CUR);
 	    args = find_args(s);
-	  } else {
-	    DOH *arg = 0;
+	  } else if (isidchar(c)) {
+	    DOH *arg = NewString("");
 	    args = NewList();
-	    arg = NewString("");
-	    if (isidchar(c)) Putc(c,arg);
-	    while ((c = Getc(s)) != EOF) {
+	    Putc(c,arg);
+	    while (((c = Getc(s)) != EOF)) {
 	      if (!isidchar(c)) {
 		Seek(s,-1,SEEK_CUR);
 		break;
 	      }
 	      Putc(c,arg);
 	    }
-	    Append(args,arg);
+	    if (Len(arg)) Append(args,arg);
 	    Delete(arg);
+	  } else {
+	    Seek(s,-1,SEEK_CUR);
 	  }
 	  if ((!args) || (!Len(args))) {
-	    Swig_error(Getfile(id),Getline(id),"No arguments given to defined()\n");
+	    /* This is not a defined() macro. */
+	    Printf(ns,"%s",id);
 	    state = 0;
 	    break;
 	  }
@@ -1023,6 +1028,26 @@ static void add_chunk(DOH *ns, DOH *chunk, int allow) {
   Clear(chunk);
 }
 
+/*
+  push/pop_imported(): helper functions for defining and undefining
+  SWIGIMPORTED (when %importing a file).
+ */
+static void
+push_imported() {
+  if (imported_depth == 0) {
+    Preprocessor_define("SWIGIMPORTED 1", 0);
+  }
+  ++imported_depth;
+}
+
+static void
+pop_imported() {
+  --imported_depth;
+  if (imported_depth == 0) {
+    Preprocessor_undef("SWIGIMPORTED");
+  }
+}
+
 /* -----------------------------------------------------------------------------
  * Preprocessor_parse()
  *
@@ -1320,8 +1345,11 @@ Preprocessor_parse(String *s)
 	  /*	  Printf(stdout,"Evaluating '%s'\n", sval); */
   	  val = Preprocessor_expr(sval,&e);
   	  if (e) {
+	    char * msg = Preprocessor_expr_error();
   	    Seek(value,0,SEEK_SET);
 	    Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Could not evaluate '%s'\n", value);
+	    if (msg)
+	      Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Error: '%s'\n", msg);
   	    allow = 0;
   	  } else {
   	    if (val == 0)
@@ -1342,8 +1370,11 @@ Preprocessor_parse(String *s)
   	    Seek(sval,0,SEEK_SET);
   	    val = Preprocessor_expr(sval,&e);
   	    if (e) {
+	      char * msg = Preprocessor_expr_error();
   	      Seek(value,0,SEEK_SET);
   	      Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Could not evaluate '%s'\n", value);
+	      if (msg)
+		Swig_warning(WARN_PP_EVALUATION,Getfile(value),Getline(value),"Error: '%s'\n", msg);
   	      allow = 0;
   	    } else {
   	      if (val)
@@ -1357,19 +1388,25 @@ Preprocessor_parse(String *s)
       } else if (Cmp(id,"include") == 0) {
   	if (((include_all) || (import_all)) && (allow)) {
   	  String *s1, *s2, *fn;
-	  char *dirname;
+	  char *dirname; int sysfile = 0;
+	  if (include_all && import_all) {
+	    Swig_warning(WARN_PP_INCLUDEALL_IMPORTALL,Getfile(s),Getline(id),"Both includeall and importall are defined: using includeall");
+	    import_all = 0;
+	  }
   	  Seek(value,0,SEEK_SET);
-  	  fn = get_filename(value);
-	  s1 = cpp_include(fn);
+  	  fn = get_filename(value, &sysfile);
+	  s1 = cpp_include(fn, sysfile);
 	  if (s1) {
 	    if (include_all) 
 	      Printf(ns,"%%includefile \"%s\" [\n", Swig_last_file());
-	    else if (import_all) 
+	    else if (import_all) {
 	      Printf(ns,"%%importfile \"%s\" [\n", Swig_last_file());
+	      push_imported();
+	    }
 
 	    /* See if the filename has a directory component */
 	    dirname = Swig_file_dirname(Swig_last_file());
-	    if (!strlen(dirname)) dirname = 0;
+	    if (sysfile || !strlen(dirname)) dirname = 0;
 	    if (dirname) {
 	      dirname[strlen(dirname)-1] = 0;   /* Kill trailing directory delimeter */
 	      Swig_push_directory(dirname);
@@ -1380,6 +1417,9 @@ Preprocessor_parse(String *s)
 	    if (dirname) {
 	      Swig_pop_directory();
 	    }
+	    if (import_all) {
+	      pop_imported();
+	    }
 	    Delete(s2);
   	  }
 	  Delete(s1);
@@ -1472,7 +1512,7 @@ Preprocessor_parse(String *s)
   	if ((Cmp(decl,"%include") == 0) || (Cmp(decl,"%import") == 0) || (Cmp(decl,"%extern") == 0)) {
   	  /* Got some kind of file inclusion directive  */
   	  if (allow) {
-  	    DOH *s1, *s2, *fn, *opt;
+  	    DOH *s1, *s2, *fn, *opt; int sysfile = 0;
 
 	    if (Cmp(decl,"%extern") == 0) {
 	      Swig_warning(WARN_DEPRECATED_EXTERN, Getfile(s),Getline(s),"%%extern is deprecated. Use %%import instead.\n");
@@ -1480,19 +1520,18 @@ Preprocessor_parse(String *s)
 	      Printf(decl,"%%import");
 	    }
 	    opt = get_options(s);
-  	    fn = get_filename(s);
-	    s1 = cpp_include(fn);
+  	    fn = get_filename(s, &sysfile);
+	    s1 = cpp_include(fn, sysfile);
 	    if (s1) {
 	      char *dirname;
   	      add_chunk(ns,chunk,allow);
   	      copy_location(s,chunk);
   	      Printf(ns,"%sfile%s \"%s\" [\n", decl, opt, Swig_last_file());
-	      if ((Cmp(decl,"%import") == 0) || (Cmp(decl,"%extern") == 0)) {
-		Preprocessor_define("WRAPEXTERN 1", 0);
-		Preprocessor_define("SWIGIMPORT 1", 0);
+	      if (Cmp(decl,"%import") == 0) {
+		push_imported();
 	      }
 	      dirname = Swig_file_dirname(Swig_last_file());
-	      if (!strlen(dirname)) dirname = 0;
+	      if (sysfile || !strlen(dirname)) dirname = 0;
 	      if (dirname) {
 		dirname[strlen(dirname)-1] = 0;   /* Kill trailing directory delimeter */
 		Swig_push_directory(dirname);
@@ -1501,9 +1540,8 @@ Preprocessor_parse(String *s)
 	      if (dirname) {
 		Swig_pop_directory();
 	      }
-	      if ((Cmp(decl,"%import") == 0) || (Cmp(decl,"%extern") == 0)) {
-		Preprocessor_undef("SWIGIMPORT");
-		Preprocessor_undef("WRAPEXTERN");
+	      if (Cmp(decl,"%import") == 0) {
+		pop_imported();
 	      }
   	      addline(ns,s2,allow);
   	      Printf(ns,"\n]");
