@@ -13,10 +13,12 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cwrap_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/cwrap.c,v 1.39 2004/06/03 22:57:27 wsfulton Exp $";
+char cvsroot_cwrap_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/cwrap.c,v 1.51 2004/12/04 08:33:02 marcelomatus Exp $";
 
 #include "swig.h"
 
+extern int    cparse_cplusplus;
+
 static Parm *nonvoid_parms(Parm *p) {
   if (p) {
     SwigType *t = Getattr(p,"type");
@@ -48,8 +50,8 @@ Swig_cparm_name(Parm *p, int i) {
  * and user defined types to pointers.
  * ----------------------------------------------------------------------------- */
 
-String *
-Swig_clocal(SwigType *t, String_or_char *name, String_or_char *value) {
+static String *
+Swig_clocal(SwigType *t, const String_or_char *name, const String_or_char *value) {
   String *decl;
 
   decl = NewString("");
@@ -136,8 +138,9 @@ Swig_wrapped_var_assign(SwigType *t, const String_or_char *name) {
  *
  * Emit all of the local variables for a list of parameters.  Returns the
  * number of parameters.
+ * Default values for the local variables are only emitted if the compact default
+ * argument behaviour is required.
  * ----------------------------------------------------------------------------- */
-
 int Swig_cargs(Wrapper *w, ParmList *p) {
   int   i;
   SwigType *pt;
@@ -148,33 +151,48 @@ int Swig_cargs(Wrapper *w, ParmList *p) {
   SwigType *altty;
   String  *type;
   int      tycode;
+  int      compactdefargs = ParmList_is_compactdefargs(p);
 
   i = 0;
+
   while (p != 0) {
     lname  = Swig_cparm_name(p,i);
     pt     = Getattr(p,"type");
     if ((SwigType_type(pt) != T_VOID)) {
       pname  = Getattr(p,"name");
-      pvalue = Getattr(p,"value");
-      altty = Getattr(p,"alttype");
+
+      /* default values only emitted if in compact default args mode */
+      if (compactdefargs)
+        pvalue = Getattr(p,"value");
+      else
+        pvalue = 0;
+
       type  = Getattr(p,"type");
+      altty = SwigType_alttype(type,0);
       tycode = SwigType_type(type);
       if (tycode == T_REFERENCE) {
 	if (pvalue) {
-	  String *defname, *defvalue;
+	  SwigType *tvalue;
+	  String *defname, *defvalue, *rvalue;
+	  rvalue = SwigType_typedef_resolve_all(pvalue);
 	  defname = NewStringf("%s_defvalue", lname);
-	  defvalue = NewStringf("%s = %s", SwigType_str(type,defname), pvalue);
+	  tvalue = Copy(type);
+	  SwigType_del_reference(tvalue);
+	  defvalue = NewStringf("%s = %s", SwigType_lstr(tvalue,defname), rvalue);
 	  Wrapper_add_localv(w,defname, defvalue, NIL);
+	  Delete(tvalue);
+	  Delete(rvalue);
 	  Delete(defname);
 	  Delete(defvalue);
 	}
-      }  else if (!pvalue && (tycode == T_POINTER)) {
+      }  else if (!pvalue && ((tycode == T_POINTER) || (tycode == T_STRING))) {
 	pvalue = (String *) "0";
       }
       if (!altty) {
 	local  = Swig_clocal(pt,lname,pvalue);
       } else {
-	local = Swig_clocal(altty,lname, pvalue);
+	local = Swig_clocal(altty,lname,pvalue);
+	Delete(altty);
       }
       Wrapper_add_localv(w,lname,local,NIL);
       i++;
@@ -246,20 +264,44 @@ Swig_cfunction_call(String_or_char *name, ParmList *parms) {
   int i = 0;
   int comma = 0;
   Parm *p = parms;
-  SwigType *pt;
   String  *nname;
 
   func = NewString("");
   nname = SwigType_namestr(name);
-  Printf(func,"%s(", nname);
-  while (p) {
-    String *pname;
-    pt = Getattr(p,"type");
 
+  /*
+    SWIG_TEMPLATE_DISAMBIGUATOR is compiler dependent (common.swg),
+      - SUN Studio 9 requires 'template', 
+      - gcc-3.4 forbids the use of 'template'.
+    the rest seems not caring very much,
+  */
+  if (SwigType_istemplate(name)) {
+    String *prefix = Swig_scopename_prefix(nname);
+    String *last = Swig_scopename_last(nname);
+    if (!prefix || Len(prefix) == 0) {
+      Printf(func,"%s(", last);
+    } else {
+      Printf(func,"%s::SWIG_TEMPLATE_DISAMBIGUATOR %s(", prefix, last);
+    }
+    Delete(prefix);
+    Delete(last);
+  } else {
+    Printf(func,"%s(", nname);
+  }
+
+
+  while (p) {
+    SwigType *pt = Getattr(p,"type");
     if ((SwigType_type(pt) != T_VOID)) {
-      if (comma) Printf(func,",");
-      pname = Swig_cparm_name(p,i);
-      Printf(func,"%s", SwigType_rcaststr(pt, pname));
+      String *pname = Swig_cparm_name(p,i);
+      String *rcaststr = SwigType_rcaststr(pt, pname);
+      if (comma) {
+	Printf(func, ",%s", rcaststr);
+      } else {
+	Printf(func, "%s", rcaststr);
+      }
+      Delete(pname);
+      Delete(rcaststr);
       comma = 1;
       i++;
     }
@@ -303,20 +345,33 @@ Swig_cmethod_call(String_or_char *name, ParmList *parms, String_or_char *self) {
   if (Strstr(func,"*this")) {
     Replaceall(func,"this", Swig_cparm_name(p,0));
   } else {
-    Replaceall(func,"this", SwigType_rcaststr(pt, Swig_cparm_name(p,0)));
+    String *rcaststr = SwigType_rcaststr(pt, Swig_cparm_name(p,0));
+    Replaceall(func,"this", rcaststr);
+    Delete(rcaststr);
   }
 
-  Printf(func,"%s(", nname);
+  /*
+    SWIG_TEMPLATE_DESIMBUAGATOR is compiler dependent (common.swg),
+      - SUN Studio 9 requires 'template', 
+      - gcc-3.4 forbids the use of 'template'.
+    the rest seems not caring very much,
+  */
+  if (SwigType_istemplate(name)) {
+    Printf(func,"SWIG_TEMPLATE_DISAMBIGUATOR %s(", nname);
+  } else {
+    Printf(func,"%s(", nname);
+  }
 
   i++;
   p = nextSibling(p);
   while (p) {
-    String *pname;
     pt = Getattr(p,"type");
     if ((SwigType_type(pt) != T_VOID)) {
+      String *pname = Swig_cparm_name(p,i);
+      String *rcaststr = SwigType_rcaststr(pt, pname);
       if (comma) Printf(func,",");
-      pname = Swig_cparm_name(p,i);
-      Printf(func,"%s", SwigType_rcaststr(pt, pname));
+      Printf(func,"%s", rcaststr);
+      Delete(rcaststr);
       comma = 1;
       i++;
     }
@@ -512,7 +567,7 @@ Swig_ref_call(Node *n, const String* lname) {
 /* -----------------------------------------------------------------------------
  * Swig_cdestructor_call()
  *
- * Creates a string that calls a C constructor function.
+ * Creates a string that calls a C destructor function.
  *
  *      free((char *) arg0);
  * ----------------------------------------------------------------------------- */
@@ -529,7 +584,7 @@ Swig_cdestructor_call(Node *n) {
 /* -----------------------------------------------------------------------------
  * Swig_cppdestructor_call()
  *
- * Creates a string that calls a C constructor function.
+ * Creates a string that calls a C destructor function.
  *
  *      delete arg0;
  * ----------------------------------------------------------------------------- */
@@ -594,6 +649,47 @@ Swig_cmemberget_call(const String_or_char *name, SwigType *t,
   return func;
 }
 
+/* -----------------------------------------------------------------------------
+ * Swig_extension_code()
+ *
+ * Generates an extension function (a function defined in %extend)
+ *
+ *        return_type function_name(parms) code
+ *
+ * ----------------------------------------------------------------------------- */
+String *
+Swig_extension_code(const String *function_name, ParmList *parms,
+		    SwigType *return_type, const String *code, int cplusplus) {
+  String *parms_str = cplusplus ? ParmList_str_defaultargs(parms) : ParmList_str(parms);
+  String *sig = NewStringf("%s(%s)", function_name, parms_str);
+  String *rt_sig = SwigType_str(return_type,sig);
+  String *body = NewStringf("static %s", rt_sig);
+  Printv(body, code, "\n", NIL);
+  Delete(parms_str);
+  Delete(sig);
+  Delete(rt_sig);
+  return body;
+}
+
+/* -----------------------------------------------------------------------------
+ * Swig_add_extension_code()
+ *
+ * Generates an extension function (a function defined in %extend) and
+ * adds it to the "wrap:code" attribute of a node
+ *
+ * See also Swig_extension_code()
+ *
+ * ----------------------------------------------------------------------------- */
+int
+Swig_add_extension_code(Node *n, const String *function_name, ParmList *parms,
+			SwigType *return_type, const String *code, int cplusplus) {
+  String *body = Swig_extension_code(function_name, parms, return_type, code, cplusplus);
+  Setattr(n,"wrap:code",body);
+  Delete(body);
+  return SWIG_OK;
+}
+
+
 /* -----------------------------------------------------------------------------
  * Swig_MethodToFunction(Node *n)
  *
@@ -660,33 +756,50 @@ Swig_MethodToFunction(Node *n, String *classname, int flags) {
   if (!(flags & CWRAP_EXTEND)) {
     Setattr(n,"wrap:action", Swig_cresult(Getattr(n,"type"),"result", Swig_cmethod_call(name,p,self)));
   } else {
-    String *code;
-    String *mangled;
-    String *membername = Swig_name_member(classname, name);
-    mangled = Swig_name_mangle(membername);
+    /* Methods with default arguments are wrapped with additional methods for each default argument,
+     * however, only one extra %extend method is generated. */
+
+    String *defaultargs = Getattr(n,"defaultargs");
+    String *code = Getattr(n,"code");
+    String *cname = Getattr(n,"classname") ? Getattr(n,"classname") : classname;
+    String *membername = Swig_name_member(cname, name);
+    String *mangled = Swig_name_mangle(membername);
 
-    code = Getattr(n,"code");
     type = Getattr(n,"type");
 
     /* Check if the method is overloaded.   If so, and it has code attached, we append an extra suffix
        to avoid a name-clash in the generated wrappers.  This allows overloaded methods to be defined
        in C. */
-
     if (Getattr(n,"sym:overloaded") && code) {
-      Append(mangled,Getattr(n,"sym:overname"));
+      Append(mangled,Getattr(defaultargs ? defaultargs : n,"sym:overname"));
     }
 
-    Setattr(n,"wrap:action", Swig_cresult(Getattr(n,"type"),"result", Swig_cfunction_call(mangled,p)));
-
     /* See if there is any code that we need to emit */
-    if (code) {
-      String *body;
-      String *tmp = NewStringf("%s(%s)", mangled, ParmList_str(p));
-      body = SwigType_str(type,tmp);
-      Delete(tmp);
-      Printv(body,code,"\n",NIL);
-      Setattr(n,"wrap:code",body);
+    if (!defaultargs && code) {
+      Swig_add_extension_code(n, mangled, p, type, code, cparse_cplusplus);
+
     }
+    if (flags & CWRAP_SMART_POINTER) {
+      int i = 0;
+      Parm *pp = p;
+      String *func = NewStringf("%s((%s*)(%s)->operator ->()", mangled, cname, 
+				Swig_cparm_name(pp,i++));
+      while ((pp = nextSibling(pp))) {
+	SwigType *pt = Getattr(pp,"type");
+	if ((SwigType_type(pt) != T_VOID)) {
+	  String *pname = Swig_cparm_name(pp,i++);
+	  String *rcaststr = SwigType_rcaststr(pt, pname);
+	  Printf(func,",%s", rcaststr);
+	  Delete(rcaststr);
+	  Delete(pname);
+	}
+      }
+      Printf(func,")");
+      Setattr(n,"wrap:action", Swig_cresult(Getattr(n,"type"),"result", func));      
+    } else {
+      Setattr(n,"wrap:action", Swig_cresult(Getattr(n,"type"),"result", Swig_cfunction_call(mangled,p)));
+    }
+    
     Delete(membername);
     Delete(mangled);
   }
@@ -771,21 +884,15 @@ Swig_ConstructorToFunction(Node *n, String *classname,
 {
   ParmList *parms;
   Parm     *prefix_args;
-  Parm     *postfix_args;
   Parm     *p;
   ParmList *directorparms;
   SwigType *type;
-  String   *membername;
-  String   *mangled;
   Node     *classNode;
   int      use_director;
   
   classNode = Swig_methodclass(n);
   use_director = Swig_directorclass(n);
 
-  membername = Swig_name_construct(classname);
-  mangled = Swig_name_mangle(membername);
-
   parms = CopyParmList(nonvoid_parms(Getattr(n,"parms")));
 
   /* Prepend the list of prefix_args (if any) */
@@ -803,49 +910,40 @@ Swig_ConstructorToFunction(Node *n, String *classname,
   } else
     directorparms = parms;
 
-  postfix_args = Getattr(n,"director:postfix_args");
-  if (postfix_args != NIL) {
-    Parm *p2, *p3, *p4;
-
-    if (prefix_args == NIL) /* no prefix args from above. */
-      directorparms = CopyParmList(parms);
-
-    if (directorparms != NIL) {
-      p2 = directorparms;
-      for ( ; nextSibling(p2); p2 = nextSibling(p2));
-      for (p3 = postfix_args; p3; p3 = nextSibling(p3)) {
-        p4 = CopyParm(p3);
-        set_nextSibling(p2, p4);
-        p2 = p4;
-      }
-    } else
-      directorparms = CopyParmList(postfix_args);
-  }
-
   type  = NewString(classname);
   SwigType_add_pointer(type);
 
   if (flags & CWRAP_EXTEND) {
+    /* Constructors with default arguments are wrapped with additional constructor methods for each default argument,
+     * however, only one extra %extend method is generated. */
+
+    String *defaultargs = Getattr(n,"defaultargs");
     String *code = Getattr(n,"code");
-    if (code) {
-      String *wrap, *s;
-      if (Getattr(n,"sym:overloaded") && code) {
-	Append(mangled,Getattr(n,"sym:overname"));
-      }
-      s = NewStringf("%s(%s)", mangled, ParmList_str(parms));
-      wrap = SwigType_str(type,s);
-      Delete(s);
-      Printv(wrap,code,"\n",NIL);
-      Setattr(n,"wrap:code",wrap);
-      Delete(wrap);
+    String *membername = Swig_name_construct(classname);
+    String *mangled = Swig_name_mangle(membername);
+
+    /* Check if the constructor is overloaded.   If so, and it has code attached, we append an extra suffix
+       to avoid a name-clash in the generated wrappers.  This allows overloaded constructors to be defined
+       in C. */
+    if (Getattr(n,"sym:overloaded") && code) {
+      Append(mangled,Getattr(defaultargs ? defaultargs : n,"sym:overname"));
     }
+
+    /* See if there is any code that we need to emit */
+    if (!defaultargs && code) {
+      Swig_add_extension_code(n, mangled, parms, type, code, cparse_cplusplus);
+    }
+
     Setattr(n,"wrap:action", Swig_cresult(type,"result", Swig_cfunction_call(mangled,parms)));
+
+    Delete(membername);
+    Delete(mangled);
   } else {
     if (cplus) {
       /* if a C++ director class exists, create it rather than the original class */
       if (use_director) {
-  	int abstract = Getattr(n, "abstract") != 0;
 	Node *parent = Swig_methodclass(n);
+  	int abstract = Getattr(parent, "abstract") != 0;
 	String *name = Getattr(parent, "sym:name");
         String* directorname = NewStringf("SwigDirector_%s", name);
 	String* action = NewString("");
@@ -898,8 +996,6 @@ Swig_ConstructorToFunction(Node *n, String *classname,
   if (directorparms != parms)
     Delete(directorparms);
   Delete(parms);
-  Delete(mangled);
-  Delete(membername);
   return SWIG_OK;
 }
 
@@ -927,10 +1023,7 @@ Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags)
     mangled = Swig_name_mangle(membername);
     code = Getattr(n,"code");
     if (code) {
-      String *s = NewStringf("void %s(%s)", mangled, ParmList_str(p));
-      Printv(s,code,"\n",NIL);
-      Setattr(n,"wrap:code",s);
-      Delete(s);
+      Swig_add_extension_code(n, mangled, p, type, code, cparse_cplusplus);
     }
     Setattr(n,"wrap:action", NewStringf("%s;\n", Swig_cfunction_call(mangled,p)));
     Delete(membername);
@@ -963,6 +1056,7 @@ Swig_MembersetToFunction(Node *n, String *classname, int flags) {
   SwigType *t;
   SwigType *ty;
   SwigType *type;
+  SwigType *void_type = NewString("void");
   String   *membername;
   String   *mangled;
   String   *self= 0;
@@ -996,19 +1090,17 @@ Swig_MembersetToFunction(Node *n, String *classname, int flags) {
   if (flags & CWRAP_EXTEND) {
     String *code = Getattr(n,"code");
     if (code) {
-      String *s = NewStringf("void %s(%s)", mangled, ParmList_str(parms));
-      Printv(s,code,"\n",NIL);
-      Setattr(n,"wrap:code",s);
-      Delete(s);
+      Swig_add_extension_code(n, mangled, parms, void_type, code, cparse_cplusplus);
     }
     Setattr(n,"wrap:action", NewStringf("%s;\n", Swig_cfunction_call(mangled,parms)));
   } else {
     Setattr(n,"wrap:action", NewStringf("%s;\n", Swig_cmemberset_call(name,type,self)));
   }
-  Setattr(n,"type","void");
+  Setattr(n,"type",void_type);
   Setattr(n,"parms", parms);
   Delete(parms);
   Delete(ty);
+  Delete(void_type);
   Delete(membername);
   Delete(mangled);
   Delete(self);
@@ -1019,7 +1111,7 @@ Swig_MembersetToFunction(Node *n, String *classname, int flags) {
 /* -----------------------------------------------------------------------------
  * Swig_MembergetToFunction()
  *
- * This function creates a C wrapper for setting a structure member.
+ * This function creates a C wrapper for getting a structure member.
  * ----------------------------------------------------------------------------- */
 
 int
@@ -1053,12 +1145,7 @@ Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   if (flags & CWRAP_EXTEND) {
     String *code = Getattr(n,"code");
     if (code) {
-      String *tmp = NewStringf("%s(%s)", mangled, ParmList_str(parms));
-      String *s = SwigType_str(ty,tmp);
-      Delete(tmp);
-      Printv(s,code,"\n",NIL);
-      Setattr(n,"wrap:code",s);
-      Delete(s);
+      Swig_add_extension_code(n, mangled, parms, ty, code, cparse_cplusplus);
     }
     Setattr(n,"wrap:action", Swig_cresult(ty,"result",Swig_cfunction_call(mangled,parms)));
   } else {
