@@ -14,7 +14,7 @@
 #include <stdarg.h>
 #include <ctype.h>
 
-char cvsroot_error_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/error.c,v 1.8 2004/08/27 21:34:41 wsfulton Exp $";
+char cvsroot_error_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/error.c,v 1.9 2004/10/18 01:50:46 marcelomatus Exp $";
 
 /* -----------------------------------------------------------------------------
  * Commentary on the warning filter.
@@ -91,11 +91,12 @@ Swig_warning(int wnum, const String_or_char *filename, int line, const char *fmt
   if (filter) {
     char    temp[32];    
     char *c;
+    char *f = Char(filter);
     sprintf(temp,"%d",wnum);
-    c = Strstr(filter,temp);
-    if (c) {
+    while(*f != '\0' && (c = strstr(f,temp))) {
       if (*(c-1) == '-') wrn = 0;     /* Warning disabled */
       if (*(c-1) == '+') wrn = 1;     /* Warning enabled */
+      f += strlen(temp);
     }
   }
   if (warnall || wrn) {
