@@ -10,9 +10,10 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_stype_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/stype.c,v 1.46 2004/05/27 11:29:39 marcelomatus Exp $";
+char cvsroot_stype_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/stype.c,v 1.54 2004/12/05 10:15:25 marcelomatus Exp $";
 
 #include "swig.h"
+#include "cparse.h"
 #include <ctype.h>
 
 /* -----------------------------------------------------------------------------
@@ -115,6 +116,9 @@ SwigType *NewSwigType(int t) {
   case T_DOUBLE:
     return NewString("double");
     break;
+  case T_COMPLEX:
+    return NewString("complex");
+    break;
   case T_CHAR:
     return NewString("char");
     break;
@@ -276,18 +280,85 @@ int SwigType_issimple(SwigType *t) {
  *     Arrays:        a().SWIGTYPE
  *     Types:         SWIGTYPE
  *     MemberPointer: m(CLASS).SWIGTYPE
- *     Enums:         enum SWIGENUM
+ *     Enums:         enum SWIGTYPE
  *
  * Note: if this function is applied to a primitive type, it returns NULL.  This
  * allows recursive application for special types like arrays.
  * ----------------------------------------------------------------------------- */
 
+#ifdef SWIG_DEFAULT_CACHE
 static Hash *default_cache = 0;
+#endif
+
+#define SWIG_NEW_TYPE_DEFAULT
+/* The new default type resolution method:
+
+1.- It preserves the original mixed types, then it goes 'backward'
+    first deleting the qualifier, then the inner types
+    
+    typedef A *Aptr;
+    const Aptr&;
+    r.q(const).Aptr       -> r.q(const).p.SWIGTYPE
+    r.q(const).p.SWIGTYPE -> r.p.SWIGTYPE
+    r.p.SWIGTYPE          -> r.SWIGTYPE
+    r.SWIGTYPE            -> SWIGTYPE
+
+
+    enum Hello {};
+    const Hello& hi;
+    r.q(const).Hello          -> r.q(const).enum SWIGTYPE
+    r.q(const).enum SWIGTYPE  -> r.enum SWIGTYPE
+    r.enum SWIGTYPE           -> r.SWIGTYPE
+    r.SWIGTYPE                -> SWIGTYPE
+
+    int a[2][4];
+    a(2).a(4).int           -> a(ANY).a(ANY).SWIGTYPE
+    a(ANY).a(ANY).SWIGTYPE  -> a(ANY).a().SWIGTYPE
+    a(ANY).a().SWIGTYPE     -> a(ANY).p.SWIGTYPE
+    a(ANY).p.SWIGTYPE       -> a(ANY).SWIGTYPE
+    a(ANY).SWIGTYPE         -> a().SWIGTYPE
+    a().SWIGTYPE            -> p.SWIGTYPE
+    p.SWIGTYPE              -> SWIGTYPE
+*/
+
+static 
+void SwigType_add_default(String *def, SwigType *nr)
+{
+  if (Strcmp(nr,"SWIGTYPE") == 0) {
+    Append(def,"SWIGTYPE");
+  } else {
+    String *q = SwigType_isqualifier(nr) ? SwigType_pop(nr) : 0;
+    if (q && Strstr(nr,"SWIGTYPE")) {
+      Append(def, nr);
+    } else {
+      String *nd = SwigType_default(nr);
+      if (nd) {
+	String *bdef = nd;
+	if (q) {
+	  bdef = NewStringf("%s%s", q, nd);
+	  if ((Strcmp(nr,bdef) == 0)) {
+	    Delete(bdef);
+	    bdef = nd;
+	  } else {
+	    Delete(nd);
+	  }
+	}
+	Append(def,bdef);
+	Delete(bdef);
+      } else {
+	Append(def,nr);
+      }
+    }
+    Delete(q);
+  }
+}
+
 
 SwigType *SwigType_default(SwigType *t) {
   String *r1, *def;
   String *r = 0;
 
+#ifdef SWIG_DEFAULT_CACHE
   if (!default_cache) default_cache = NewHash();
   
   r = Getattr(default_cache,t);
@@ -295,6 +366,7 @@ SwigType *SwigType_default(SwigType *t) {
     if (Strcmp(r,t) == 0) return 0;
     return Copy(r);
   }
+#endif
  
   if (SwigType_isvarargs(t)) {
     return 0;
@@ -306,47 +378,96 @@ SwigType *SwigType_default(SwigType *t) {
     r = r1;
   }
   if (SwigType_isqualifier(r)) {
+    String *q;
     if (r == t) r = Copy(t);
-    do {
-      Delete(SwigType_pop(r));
-    } while (SwigType_isqualifier(r));
+    q = SwigType_pop(r);
+    if (Strstr(r,"SWIGTYPE")) {
+      Delete(q);
+      def = r;
+      return def;
+    }
   }
-  if (SwigType_ispointer(r)) {
+  if (Strcmp(r,"p.SWIGTYPE") == 0) {
+    def = NewString("SWIGTYPE");
+  } else if (SwigType_ispointer(r)) {
+#ifdef SWIG_NEW_TYPE_DEFAULT
+    SwigType *nr = Copy(r);
+    SwigType_del_pointer(nr);
+    def = NewStringf("p.");
+    SwigType_add_default(def, nr);
+    Delete(nr);
+#else
     def = NewString("p.SWIGTYPE");
+#endif
+  } else if (Strcmp(r,"r.SWIGTYPE") == 0) {
+    def = NewString("SWIGTYPE");
   } else if (SwigType_isreference(r)) {
+#ifdef SWIG_NEW_TYPE_DEFAULT
+    SwigType *nr = Copy(r);
+    SwigType_del_reference(nr);
+    def = NewStringf("r.");
+    SwigType_add_default(def, nr);
+    Delete(nr);
+#else
     def = NewString("r.SWIGTYPE");
+#endif
   } else if (SwigType_isarray(r)) {
-    if (Strcmp(r,"a(ANY).SWIGTYPE") == 0) {
+    if (Strcmp(r,"a().SWIGTYPE") == 0) {
+      def = NewString("p.SWIGTYPE");
+    } else if (Strcmp(r,"a(ANY).SWIGTYPE") == 0) {
       def = NewString("a().SWIGTYPE");
     } else {
       int i, empty = 0;
       int ndim = SwigType_array_ndim(r);
+      SwigType *nr = Copy(r);
       for (i = 0; i < ndim; i++) {
 	String *dim = SwigType_array_getdim(r,i);
 	if (!Len(dim)) {
-	  empty = 1;
+	  char *c = Char(nr);
+	  empty = strstr(c,"a(ANY).") != c;
 	}
 	Delete(dim);
       }
       if (empty) {
-	def = NewString("a().SWIGTYPE");
+	def = NewString("a().");
       } else {
-	def = NewString("a(ANY).SWIGTYPE");
+	def = NewString("a(ANY).");
       }
+#ifdef SWIG_NEW_TYPE_DEFAULT
+      SwigType_del_array(nr);
+      SwigType_add_default(def, nr);
+      Delete(nr);
+#else
+      Append(def,"SWIGTYPE");
+#endif
     }
   } else if (SwigType_ismemberpointer(r)) {
-    def = NewString("m(CLASS).SWIGTYPE");
+    if (Strcmp(r,"m(CLASS).SWIGTYPE") == 0) {
+      def = NewString("p.SWIGTYPE");
+    } else {
+      def = NewString("m(CLASS).SWIGTYPE");
+    }
   } else if (SwigType_isenum(r)) {
-    def = NewString("enum SWIGTYPE");
+    if (Strcmp(r,"enum SWIGTYPE") == 0) {
+      def = NewString("SWIGTYPE");
+    } else {
+      def = NewString("enum SWIGTYPE");
+    }
   } else {
     def = NewString("SWIGTYPE");
   }
   if (r != t) Delete(r);
-  Setattr(default_cache,t,Copy(def));
+#ifdef SWIG_DEFAULT_CACHE
+  /* The cache produces strange results, see enum_template.i case */
+  Setattr(default_cache,t,Copy(def)); 
+#endif
   if (Strcmp(def,t) == 0) {
     Delete(def);
     def = 0;
   }
+
+  /* Printf(stderr,"type : def %s : %s\n", t, def);  */
+
   return def;
 }
 
@@ -512,6 +633,7 @@ SwigType_ltype(SwigType *s) {
   List *elements;
   int nelements, i;
   int firstarray = 1;
+  int notypeconv = 0;
 
   result = NewString("");
   tc = Copy(s);
@@ -522,7 +644,7 @@ SwigType_ltype(SwigType *s) {
   if (SwigType_issimple(tc)) {
     /* Resolve any typedef definitions */
     td = SwigType_typedef_resolve(tc);
-    if (td && (SwigType_isconst(td) || SwigType_isarray(td) || SwigType_isenum(td) || SwigType_isreference(td))) {
+    if (td && (SwigType_isconst(td) || SwigType_isarray(td) || SwigType_isreference(td))) {
       /* We need to use the typedef type */
       Delete(tc);
       tc = td;
@@ -532,9 +654,14 @@ SwigType_ltype(SwigType *s) {
   }
   elements = SwigType_split(tc);
   nelements = Len(elements);
+
   /* Now, walk the type list and start emitting */
   for (i = 0; i < nelements; i++) {
     element = Getitem(elements,i);
+    /* when we see a function, we need to preserve the following types */
+    if (SwigType_isfunction(element)) {
+      notypeconv = 1;
+    }
     if (SwigType_isqualifier(element)) {
       /* Do nothing. Ignore */
     } else if (SwigType_ispointer(element)) {
@@ -544,13 +671,26 @@ SwigType_ltype(SwigType *s) {
       Append(result,element);
       firstarray = 0;
     } else if (SwigType_isreference(element)) {
-      Append(result,"p.");
+      if (notypeconv) {
+	Append(result,element);
+      } else {
+	Append(result,"p.");
+      }
       firstarray = 0;
     } else if (SwigType_isarray(element) && firstarray) {
-      Append(result,"p.");
+      if (notypeconv) {
+	Append(result,element);
+      } else {
+	Append(result,"p.");
+      }
       firstarray = 0;
     } else if (SwigType_isenum(element)) {
-      Append(result,"int");
+      int anonymous_enum = (Cmp(element,"enum ") == 0); 
+      if (notypeconv || !anonymous_enum) {
+	Append(result,element);
+      } else {
+	Append(result,"int");
+      }
     } else {
       Append(result,element);
     }
@@ -615,17 +755,17 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
     rs = s;
   }
 
-  if (SwigType_issimple(rs)) {
-    td = SwigType_typedef_resolve(rs);
-    if ((td) && (SwigType_isconst(td) || SwigType_isarray(td) || SwigType_isreference(td))) {
+  td = SwigType_typedef_resolve(rs);
+  if (td) {
+    if ((SwigType_isconst(td) || SwigType_isarray(td) || SwigType_isreference(td))) {
       elements = SwigType_split(td);
-    } else if (td && SwigType_isenum(td)) {
+    } else if (SwigType_isenum(td)) {
       elements = SwigType_split(rs);
       clear = 0;
     } else {
       elements = SwigType_split(rs);
     } 
-    if (td) Delete(td);
+    Delete(td);
   } else {
     elements = SwigType_split(rs);
   }
@@ -701,6 +841,7 @@ String *SwigType_rcaststr(SwigType *s, const String_or_char *name) {
       Delete(parms);
     } else if (SwigType_isenum(element)) {
       String *bs = SwigType_namestr(element);
+      Insert(result,0," ");
       Insert(result,0,bs);
       Delete(bs);
       clear = 0;
@@ -762,11 +903,14 @@ String *SwigType_manglestr_default(SwigType *s) {
   char *c;
   String *result,*base;
   SwigType *lt;
-  SwigType *ss = 0;
+  SwigType *ss = SwigType_typedef_resolve_all(s);
+  s = ss;
 
   if (SwigType_istemplate(s)) {
-    ss = SwigType_typedef_resolve_all(s);
+    String *st = ss;
+    ss = Swig_cparse_template_deftype(st, 0);
     s = ss;
+    Delete(st);
   }
   lt = SwigType_ltype(s);
   result = SwigType_prefix(lt);
@@ -786,6 +930,7 @@ String *SwigType_manglestr_default(SwigType *s) {
   Replace(base,"struct ","", DOH_REPLACE_ANY);     /* This might be problematic */
   Replace(base,"class ","", DOH_REPLACE_ANY);
   Replace(base,"union ","", DOH_REPLACE_ANY);
+  Replace(base,"enum ","", DOH_REPLACE_ANY);
 
   c = Char(base);
   while (*c) {
