@@ -9,7 +9,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_symbol_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/symbol.c,v 1.10 2004/04/09 19:23:01 cheetah Exp $";
+char cvsroot_symbol_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/symbol.c,v 1.20 2004/12/01 00:56:35 marcelomatus Exp $";
 
 #include "swig.h"
 #include "swigwarn.h"
@@ -178,12 +178,13 @@ void
 Swig_symbol_dump_symtable() {
   Printf(stdout, "DUMPING SYMTABLE start =======================================\n");
   {
-  Hash* cst = Getattr(current_symtab, "csymtab");
-  Swig_print_tree(cst);
-  Printf(stdout, "DUMPING SYMTABLE middle =======================================\n");
-  Swig_print_tree(Getattr(cst, "NumSpace"));
+    Hash* cst = Getattr(current_symtab, "csymtab");
+    Swig_print_tree(cst);
+    /*
+    Swig_print_tree(Getattr(cst, "NumSpace"));
+    */
   }
-  Printf(stdout, "DUMPING SYMTABLE end =======================================\n");
+  Printf(stdout, "DUMPING SYMTABLE end   =======================================\n");
 }
 #endif
 
@@ -675,20 +676,32 @@ Swig_symbol_add(String_or_char *symname, Node *n) {
     if (Cmp(nstorage,"typedef") == 0) {
       return c;
     }
+
     /* Okay. Walk down the list of symbols and see if we get a declarator match */
-    cn = c;
-    pn = 0;
-    while (cn) {
-      decl = Getattr(cn,"decl");
-      if (!(u1 || u2)) {
-	if (Cmp(ndecl,decl) == 0) {
-	  /* Declarator conflict */
-	  return cn;
-	}
+    {
+      int n_template = (Strcmp(nodeType(n),"template") == 0) && (Strcmp(Getattr(n,"templatetype"),"cdecl") == 0);
+      int n_plain_cdecl = (Strcmp(nodeType(n),"cdecl") == 0);
+      cn = c;
+      pn = 0;
+      while (cn) {
+        decl = Getattr(cn,"decl");
+        if (!(u1 || u2)) {
+          if (Cmp(ndecl,decl) == 0) {
+            /* Declarator conflict */
+            /* Now check we don't have a non-templated function overloaded by a templated function with same params,
+             * eg void foo(); template<typename> void foo(); */
+            int cn_template = (Strcmp(nodeType(cn),"template") == 0) && (Strcmp(Getattr(cn,"templatetype"),"cdecl") == 0);
+            int cn_plain_cdecl = (Strcmp(nodeType(cn),"cdecl") == 0);
+            if (!((n_template && cn_plain_cdecl) || (cn_template && n_plain_cdecl))) {
+              /* found a conflict */
+              return cn;
+            }
+          }
+        }
+        cl = cn;
+        cn = Getattr(cn,"sym:nextSibling");
+        pn++;
       }
-      cl = cn;
-      cn = Getattr(cn,"sym:nextSibling");
-      pn++;
     }
     /* Well, we made it this far.  Guess we can drop the symbol in place */
     Setattr(n,"sym:symtab",current_symtab);
@@ -1059,7 +1072,9 @@ Swig_symbol_cscope(String_or_char *name, Symtab *symtab) {
 /* -----------------------------------------------------------------------------
  * Swig_symbol_remove()
  *
- * Remove a symbol 
+ * Remove a symbol. If the symbol is an overloaded function and the symbol removed
+ * is not the last in the list of overloaded functions, then the overloaded
+ * names (sym:overname attribute) are changed to start from zero, eg __SWIG_0.
  * ----------------------------------------------------------------------------- */
 
 void
@@ -1068,6 +1083,7 @@ Swig_symbol_remove(Node *n) {
   String  *symname;
   Node    *symprev;
   Node    *symnext;
+  Node    *fixovername = 0;
   symtab  = Getattr(n,"sym:symtab");        /* Get symbol table object */
   symtab  = Getattr(symtab,"symtab");       /* Get actual hash table of symbols */
   symname = Getattr(n,"sym:name");
@@ -1078,6 +1094,7 @@ Swig_symbol_remove(Node *n) {
   if (symprev) {
     if (symnext) {
       Setattr(symprev,"sym:nextSibling",symnext);
+      fixovername = symprev; /* fix as symbol to remove is somewhere in the middle of the linked list */
     } else {
       Delattr(symprev,"sym:nextSibling");
     }
@@ -1085,6 +1102,7 @@ Swig_symbol_remove(Node *n) {
     /* If no previous symbol, see if there is a next symbol */
     if (symnext) {
       Setattr(symtab,symname,symnext);
+      fixovername = symnext; /* fix as symbol to remove is at head of linked list */
     } else {
       Delattr(symtab,symname);
     }
@@ -1103,34 +1121,29 @@ Swig_symbol_remove(Node *n) {
   Delattr(n,"sym:overname");
   Delattr(n,"csym:previousSibling");
   Delattr(n,"sym:overloaded");
-  return;
+  n = 0;
 
-#if 0
-  symtab  = Getattr(n,"sym:symtab");        /* Get symbol table object */
-  symtab  = Getattr(symtab,"csymtab");       /* Get actual hash table of symbols */
-  symprev = Getattr(n,"csym:previousSibling");
-  symnext = Getattr(n,"csym:nextSibling");
+  if (fixovername) {
+    Node *nn = fixovername;
+    Node *head = fixovername;
+    int pn = 0;
 
-  /* If previous symbol, just fix the links */
-  if (symprev) {
-    if (symnext) {
-      Setattr(symprev,"csym:nextSibling",symnext);
-    } else {
-      Delattr(symprev,"csym:nextSibling");
+    /* find head of linked list */
+    while (nn) {
+      head = nn;
+      nn = Getattr(nn, "sym:previousSibling");
     }
-  } else {
-    /* If no previous symbol, see if there is a next symbol */
-    if (symnext) {
-      Setattr(symtab,Getattr(n,"name"),symnext);
-    } else {
-      Delattr(symtab,Getattr(n,"name"));
+
+    /* adjust all the sym:overname strings to start from 0 and increment by one */
+    nn = head;
+    while (nn) {
+      assert(Getattr(nn,"sym:overname"));
+      Delattr(nn,"sym:overname");
+      Setattr(nn,"sym:overname", NewStringf("__SWIG_%d", pn));
+      pn++;
+      nn = Getattr(nn,"sym:nextSibling");
     }
   }
-
-  Delattr(n,"sym:symtab");
-  Delattr(n,"csym:previousSibling");
-  Delattr(n,"csym:nextSibling");
-#endif
 }
 
 /* -----------------------------------------------------------------------------
@@ -1206,12 +1219,12 @@ Swig_symbol_type_qualify(SwigType *t, Symtab *st) {
 	Printf(qprefix,"<(");
 	for (ti = First(targs); ti.item;) {
 	  String *qparm = Swig_symbol_type_qualify(ti.item,st);
-	  /*	  Printf(stdout,"qparm = '%s', tparm = '%s'\n", qparm, ti.item);*/
 	  while (1) {
 	    /* It is possible for an integer to show up here.  If so, we need to evaluate it */
-	    {
-	      Node *nn = Swig_symbol_clookup(qparm,st);
-	      if ((nn) && (Strcmp(nodeType(nn),"cdecl") == 0)) {
+	    Node *nn = Swig_symbol_clookup(qparm,st);
+	    if (nn) {
+	      SwigType *nt = nodeType(nn);
+	      if (Strcmp(nt,"cdecl") == 0) {
 		String *nv = Getattr(nn,"value");
 		if (nv) {
 		  Clear(qparm);
@@ -1219,7 +1232,7 @@ Swig_symbol_type_qualify(SwigType *t, Symtab *st) {
 		} else {
 		  break;
 		}
-	      } else if ((nn) && (Strcmp(nodeType(nn),"enumitem") == 0)) {
+	      } else if (Strcmp(nt,"enumitem") == 0) {
 		String *qn = Swig_symbol_qualified(nn);
 		if (Len(qn)) {
 		  Append(qn,"::");
@@ -1232,6 +1245,8 @@ Swig_symbol_type_qualify(SwigType *t, Symtab *st) {
 	      } else {
 		break;
 	      }
+	    } else {
+	      break;
 	    }
 	  }
 	  Append(qprefix,qparm);
@@ -1287,6 +1302,7 @@ SwigType *Swig_symbol_typedef_reduce(SwigType *ty, Symtab *tab) {
   SwigType *prefix, *base;
   Node *n;
 
+
   base = SwigType_base(ty);
   prefix = SwigType_prefix(ty);
 
@@ -1309,10 +1325,24 @@ SwigType *Swig_symbol_typedef_reduce(SwigType *ty, Symtab *tab) {
   }
   if (Strcmp(nodeType(n),"cdecl") == 0) {
     String *storage = Getattr(n,"storage");
-    if (Strcmp(storage,"typedef") == 0) {
+    if (storage && (Strcmp(storage,"typedef") == 0)) {
       SwigType *decl;
       SwigType *rt;
+      SwigType *qt;
+      Symtab *ntab;
       SwigType *nt = Copy(Getattr(n,"type"));
+      
+      /* Fix for case 'typedef struct Hello hello;' */
+      {	
+	const char* dclass[3] = {"struct ", "union ", "class "};
+	int i;
+	for (i=0; i<3; i++) {
+	  char * c = Char(nt);
+	  if (strstr(c, dclass[i]) == c) {
+	    Replace(nt,dclass[i],"", DOH_REPLACE_FIRST);
+	  }
+	}
+      }
       decl = Getattr(n,"decl");
       if (decl) {
 	SwigType_push(nt,decl);
@@ -1320,9 +1350,52 @@ SwigType *Swig_symbol_typedef_reduce(SwigType *ty, Symtab *tab) {
       SwigType_push(nt,prefix);
       Delete(base);
       Delete(prefix);
-      rt = Swig_symbol_typedef_reduce(nt, Getattr(n,"sym:symtab"));
+      ntab = Getattr(n,"sym:symtab");
+      rt = Swig_symbol_typedef_reduce(nt, ntab);
+      qt = Swig_symbol_type_qualify(rt, ntab);
+      if (SwigType_istemplate(qt)) {
+	Iterator pi;
+	Parm *p;
+	List *parms = SwigType_parmlist(qt);
+	String *tprefix = SwigType_templateprefix(qt);
+	String *tsuffix = SwigType_templatesuffix(qt);
+	String *qprefix = SwigType_typedef_qualified(tprefix);
+	Printv(qprefix,"<(",NIL);
+	pi = First(parms);
+	while ((p = pi.item)) {
+	  String *np;
+	  String *tp = 0;
+	  String *qp = Swig_symbol_type_qualify(p, ntab);
+	  Node *n = Swig_symbol_clookup(qp,ntab);
+	  if (n) {
+	    String *qual = Swig_symbol_qualified(n);
+	    np = Copy(Getattr(n,"name"));
+	    tp = np;
+	    if (qual) {
+	      Insert(np,0,"::");
+	      Insert(np,0,qual);
+	      Delete(qual);
+	    }
+	  } else {
+	    np = qp;
+	  }
+	  Append(qprefix,np);
+	  pi= Next(pi);
+	  if (pi.item) {
+	    Append(qprefix,",");
+	  }
+	  Delete(qp);
+	  Delete(tp);
+	}
+	Append(qprefix,")>");
+	Insert(tsuffix, 0, qprefix);
+	Delete(qt);
+	qt = tsuffix;
+      }      
       Delete(nt);
-      return rt;
+      Delete(rt);
+      /* Printf(stderr,"reduce  %s %s\n", ty, qt); */
+      return qt;
     }
   }
   Delete(base);
