@@ -9,7 +9,7 @@
  * See the file LICENSE for information on usage and redistribution.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_typemap_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typemap.c,v 1.33 2004/08/24 21:48:46 wsfulton Exp $";
+char cvsroot_typemap_c[] = "$Header: /cvsroot/swig/SWIG/Source/Swig/typemap.c,v 1.42 2004/12/01 01:01:03 marcelomatus Exp $";
 
 #include "swig.h"
 #include "cparse.h"
@@ -46,6 +46,33 @@ static void replace_embedded_typemap(String *s);
 static Hash *typemaps[MAX_SCOPE];
 static int   tm_scope = 0;
 
+static Hash* get_typemap(int tm_scope, SwigType* type)
+{
+  Hash *tm  = 0;
+  SwigType* dtype = 0;
+  if (SwigType_istemplate(type)) {
+    dtype = Swig_cparse_template_deftype(type,0);
+    /* Printf(stderr,"gettm %s %s\n", type, dtype);*/
+    type = dtype;
+  }
+  tm = Getattr(typemaps[tm_scope],type);
+  if (dtype) Delete(dtype);
+  return tm;
+}
+
+static void set_typemap(int tm_scope, SwigType* type, Hash* tm)
+{
+  SwigType* dtype = 0;
+  if (SwigType_istemplate(type)) {
+    dtype = Swig_cparse_template_deftype(type,0);
+    /* Printf(stderr,"settm %s %s\n", type, dtype);*/
+    type = dtype; 
+  }
+  Setattr(typemaps[tm_scope],Copy(type),tm);
+  if (dtype) Delete(dtype);
+}
+
+
 /* -----------------------------------------------------------------------------
  * Swig_typemap_init()
  *
@@ -135,10 +162,10 @@ Swig_typemap_register(const String_or_char *op, ParmList *parms, String_or_char
   pname = Getattr(parms,"name");
 
   /* See if this type has been seen before */
-  tm = Getattr(typemaps[tm_scope],type);
+  tm = get_typemap(tm_scope,type);
   if (!tm) {
     tm = NewHash();
-    Setattr(typemaps[tm_scope],Copy(type),tm);
+    set_typemap(tm_scope,type,tm);
     Delete(tm);
   }
   if (pname) {
@@ -216,7 +243,7 @@ Swig_typemap_get(SwigType *type, String_or_char *name, int scope) {
   Hash *tm, *tm1;
   /* See if this type has been seen before */
   if ((scope < 0) || (scope > tm_scope)) return 0;
-  tm = Getattr(typemaps[scope],type);
+  tm = get_typemap(scope,type);
   if (!tm) {
     return 0;
   }
@@ -368,10 +395,11 @@ Swig_typemap_apply(ParmList *src, ParmList *dest) {
   }
 
   /* make sure a typemap node exists for the last destination node */
-  tm = Getattr(typemaps[tm_scope],Getattr(lastdp,"type"));
+  type = Getattr(lastdp,"type");
+  tm = get_typemap(tm_scope,type);
   if (!tm) {
     tm = NewHash();
-    Setattr(typemaps[tm_scope],Getattr(lastdp,"type"),tm);
+    set_typemap(tm_scope,type,tm);
     Delete(tm);
   }
   name = Getattr(lastdp,"name");
@@ -396,6 +424,21 @@ Swig_typemap_apply(ParmList *src, ParmList *dest) {
     /* See if there is a matching typemap in this scope */
     sm = Swig_typemap_get(type,name,ts);
 
+    /* if there is not matching, look for a typemap in the
+       original typedef, if any, like in:
+
+         typedef unsigned long size_t;
+	...
+	%apply(size_t) {my_size};  ==>  %apply(unsigned long) {my_size};
+    */
+    if (!sm) {
+      SwigType *ntype = SwigType_typedef_resolve(type);
+      if (ntype && (Cmp(ntype,type) != 0)) {
+	sm = Swig_typemap_get(ntype,name,ts);
+      }
+      Delete(ntype);
+    }
+    
     if (sm) {
       /* Got a typemap.  Need to only merge attributes for methods that match our signature */
       Iterator ki;
@@ -463,7 +506,7 @@ Swig_typemap_clear_apply(Parm *parms) {
     }
     p = np;
   }
-  tm = Getattr(typemaps[tm_scope],Getattr(lastp,"type"));
+  tm = get_typemap(tm_scope,Getattr(lastp,"type"));
   if (!tm) {
     Delete(tsig);
     return;
@@ -531,7 +574,7 @@ Swig_typemap_search(const String_or_char *op, SwigType *type, const String_or_ch
     ctype = type;
     while (ctype) {
       /* Try to get an exact type-match */
-      tm = Getattr(typemaps[ts],ctype);
+      tm = get_typemap(ts,ctype);
       if (tm && cname) {
 	tm1 = Getattr(tm,cname);
 	if (tm1) {
@@ -552,7 +595,7 @@ Swig_typemap_search(const String_or_char *op, SwigType *type, const String_or_ch
 	if (!noarrays) {
 	  noarrays = strip_arrays(ctype);
 	}
-	tma = Getattr(typemaps[ts],noarrays);
+	tma = get_typemap(ts,noarrays);
 	if (tma && cname) {
 	  tm1 = Getattr(tma,cname);
 	  if (tm1) {
@@ -598,7 +641,7 @@ Swig_typemap_search(const String_or_char *op, SwigType *type, const String_or_ch
 
     primitive = SwigType_default(type);
     while (primitive) {
-      tm = Getattr(typemaps[ts],primitive);
+      tm = get_typemap(ts,primitive);
       if (tm && cname) {
 	tm1 = Getattr(tm,cname);
 	if (tm1) {
@@ -703,7 +746,7 @@ int check_locals(ParmList *p, const char *s) {
 }
 
 static
-void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *pname, String *lname, int index) 
+void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, SwigType *rtype, String *pname, String *lname, int index) 
 {
   char var[512];
   char *varname;
@@ -735,7 +778,7 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
     String *size;
     int ndim;
     int i;
-    if (!SwigType_isarray(type)) type = ftype;
+    if (SwigType_array_ndim(type) != SwigType_array_ndim(ftype)) type = ftype;
     ndim = SwigType_array_ndim(type);
     size = NewString("");
     for (i = 0; i < ndim; i++) {
@@ -768,7 +811,7 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
 
   /* Type-related stuff */
   {
-    SwigType *star_type, *amp_type, *base_type;
+    SwigType *star_type, *amp_type, *base_type, *lex_type;
     SwigType *ltype, *star_ltype, *amp_ltype;
     String *mangle, *star_mangle, *amp_mangle, *base_mangle, *base_name;
     String *descriptor, *star_descriptor, *amp_descriptor;
@@ -967,6 +1010,13 @@ void typemap_replace_vars(String *s, ParmList *locals, SwigType *type, String *p
     Delete(base_mangle);
     Delete(base_type);
     Delete(base_name);
+ 
+    lex_type = SwigType_base(rtype);
+    if (index == 1)
+      Replace(s,"$lextype", lex_type, DOH_REPLACE_ANY);
+    strcpy(varname,"lextype");
+    Replace(s,var,lex_type,DOH_REPLACE_ANY);
+    Delete(lex_type);
   }
   
   /* Replace any $n. with (&n)-> */
@@ -997,8 +1047,10 @@ static void typemap_locals(DOHString *s, ParmList *l, Wrapper *f, int argnum) {
   p = l;
   while (p) {
     SwigType *pt = Getattr(p,"type");
+    SwigType *at = SwigType_alttype(pt, 1);
     String   *pn = Getattr(p,"name");
     String   *value = Getattr(p,"value");
+    if (at) pt = at;
     if (pn) {
       if (Len(pn) > 0) {
 	String *str;
@@ -1034,6 +1086,7 @@ static void typemap_locals(DOHString *s, ParmList *l, Wrapper *f, int argnum) {
       }
     }
     p = nextSibling(p);
+    Delete(at);
   }
 }
 
@@ -1067,9 +1120,9 @@ String *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_
 
   /* This is wrong.  It replaces locals in place.   Need to fix this */
   if (mtype && SwigType_isarray(mtype)) {
-    typemap_replace_vars(s,locals,mtype,pname,lname,1);
+    typemap_replace_vars(s,locals,mtype,type,pname,lname,1);
   } else {
-    typemap_replace_vars(s,locals,type,pname,lname,1);
+    typemap_replace_vars(s,locals,type,type,pname,lname,1);
   }
 
   if (locals && f) {
@@ -1102,9 +1155,6 @@ String *Swig_typemap_lookup(const String_or_char *op, SwigType *type, String_or_
  * Attach one or more typemaps to a node
  * ----------------------------------------------------------------------------- */
 
-Node *
-Swig_cparse_template_locate(String *name, Parm *tparms);
-
 String *Swig_typemap_lookup_new(const String_or_char *op, Node *node, const String_or_char *lname, Wrapper *f)
 {
   SwigType *type;
@@ -1156,9 +1206,9 @@ String *Swig_typemap_lookup_new(const String_or_char *op, Node *node, const Stri
   }
 
   if (mtype && SwigType_isarray(mtype)) {
-    typemap_replace_vars(s,locals,mtype,pname,(char *) lname,1);
+    typemap_replace_vars(s,locals,mtype,type,pname,(char *) lname,1);
   } else {
-    typemap_replace_vars(s,locals,type,pname,(char *) lname,1);
+    typemap_replace_vars(s,locals,type,type,pname,(char *) lname,1);
   }
 
   if (locals && f) {
@@ -1352,10 +1402,10 @@ Swig_typemap_attach_parms(const String_or_char *op, ParmList *parms, Wrapper *f)
       mtype = Getattr(p,"tmap:match");
 
       if (mtype) {
-	typemap_replace_vars(s,locals, mtype,pname,lname,i+1);
+	typemap_replace_vars(s,locals,mtype,type,pname,lname,i+1);
 	Delattr(p,"tmap:match");
       } else {
-	typemap_replace_vars(s,locals, type,pname,lname,i+1);
+	typemap_replace_vars(s,locals,type,type,pname,lname,i+1);
       }
 
       if (checkAttribute(tm,"type","SWIGTYPE")) {
@@ -1396,6 +1446,9 @@ Swig_typemap_attach_parms(const String_or_char *op, ParmList *parms, Wrapper *f)
 
     /* Look for code fragments */
     Swig_typemap_emit_code_fragments(op,firstp);
+
+    /* increase argnum to consider numinputs */
+    argnum += nmatch - 1;
   }
 }
 
@@ -1411,6 +1464,7 @@ Swig_typemap_attach_parms(const String_or_char *op, ParmList *parms, Wrapper *f)
  *
  *   $TYPEMAP(in, $1=int x, $input=y, ...)
  *
+ * Note: this was added as an experiment and could be removed
  * ----------------------------------------------------------------------------- */
 
 /* Splits the arguments of an embedded typemap */
